from .dates import Dates
from .markdown import Chat, Format
from .networking import Networking
from .osWrapper import environ, environ_list
from .sleeper import Sleeper
from .subprocessWrapper import Run



import unittest

from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults().enable_headless()


class Test(unittest.TestCase):
    try:
        if browser.run():

            def test_autosave(self):
                if browser.get('http://bing.com'):
                    self.assertTrue(browser.autosave_cookies())

                    browser.quit()

    except:
        pass


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.instagram.client_browser import InstagramBrowserClient

c = InstagramBrowserClient(headless=True)
c.start()


class InstagramClientTest(unittest.TestCase):

    def test(self):
        if c.is_ready():
            c.browser.get(c.urls.login_page)

        c.browser.quit()


if __name__ == '__main__':
    unittest.main()



import unittest
import json
import asyncio

from automon.integrations.swimlaneWrapper.client import SwimlaneClientRest

client = SwimlaneClientRest()


class MyTestCase(unittest.TestCase):
    def test_login(self):
        if asyncio.run(client.is_ready()):
            if asyncio.run(client.login_token()):
                appId = client.config.appId

                key = 'a1qio'  # base64
                value = dict(
                    key='value',
                    key2='value2',
                )

                value = 'value'

                record_new = asyncio.run(
                    client.record_create(
                        appId=appId,
                        key=key,
                        value=value)
                )

                self.assertTrue(record_new)

                record_id = record_new.get('id')

                record_get = asyncio.run(
                    client.record_get(appId=appId, id=record_id))

                self.assertTrue(record_get)

                pass


if __name__ == '__main__':
    unittest.main()



import os

from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.ERROR)


class ShodanConfig:
    token = os.getenv('SHODAN_API')

    def __init__(self):
        self.token = os.getenv('SHODAN_API')

        if not self.token:
            logger.error(f'Missing SHODAN_API')


class Shodan:
    """Get any Shodan information"""

    def __init__(self, api_key: str = None):
        self.api_key = api_key if api_key else ShodanConfig.token

    # TODO: add shodan geoip
    def request(self):
        pass



from automon import log

from .datatypes import AbstractDataType

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class Asset(AbstractDataType):
    def __init__(self, asset: dict = {}):
        self.__dict__.update(asset)



import unittest
import asyncio

from automon.integrations.google.auth import GoogleAuthConfig


class MyTestCase(unittest.TestCase):
    async def test_something(self):
        test = GoogleAuthConfig()
        if await test.Credentials():
            self.assertTrue(asyncio.run(test.Credentials()))


if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio

from automon.integrations.swimlaneWrapper.client import SwimlaneClientRest

client = SwimlaneClientRest()


class MyTestCase(unittest.TestCase):
    def test_login(self):
        if asyncio.run(client.is_ready()):
            if client.config.jwt_token:
                self.assertTrue(asyncio.run(
                    client.login_token()
                ))

                self.assertFalse(asyncio.run(
                    client.create_auth_token()
                ))

        pass


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.mac import os_is_mac
from automon.integrations.mac.wdutil import WdutilClient, WdutilConfig


class MyTestCase(unittest.TestCase):
    if os_is_mac():
        if WdutilConfig().is_ready():

            def test_something(self):
                self.assertTrue(
                    WdutilClient().is_ready()
                )


if __name__ == '__main__':
    unittest.main()



from automon import environ
from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.logging.DEBUG)


class DatadogConfigRest(object):
    api_key: str

    def __init__(self, host: str = 'https://api.datadoghq.com', api_key: str = None, app_key: str = None):
        self.host = host or environ('DD_SITE')
        self.host_log = 'https://http-intake.logs.us5.datadoghq.com'
        self.api_key = api_key or environ('DD_API_KEY')
        self.app_key = app_key or environ('DD_APP_KEY')

    async def is_ready(self):
        if self.host and self.api_key and self.app_key:
            return True
        logger.error(f'missing DD_SITE DD_API_KEY DD_APP_KEY')

    async def headers(self):
        if await self.is_ready():
            return {
                'DD-API-KEY': f'{self.api_key}',
                'DD-APPLICATION-KEY': f'{self.app_key}',
                'Accept': 'application/json',
                'Content-Type': 'application/json',
            }



import unittest
import asyncio

from automon.integrations.openTelemetryWrapper import OpenTelemetryClient


class MyTestCase(unittest.TestCase):
    client = OpenTelemetryClient()

    def test_client(self):
        self.assertTrue(asyncio.run(
            self.client.is_ready()
        ))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon import log
from automon.integrations.sentryio.client import SentryClient

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)
sentry = SentryClient()


class CallbackTest(unittest.TestCase):

    def test_sentry(self):
        self.assertIsNone(logger.info('test'))
        self.assertIsNone(logger.debug('test'))
        self.assertIsNone(logger.error('test'))
        self.assertIsNone(logger.warning('test'))
        self.assertIsNone(logger.critical('test'))


if __name__ == '__main__':
    unittest.main()



from automon import Networking


class V2(object):

    def __init__(self, host):
        self.endpoint = host

    @property
    def api(self):
        self.endpoint += '/api/v2'
        return self

    @property
    def logs(self):
        self.endpoint += '/logs'
        return self



from enum import Enum


class Api(object):
    api = f'api'


class Auth(object):
    api = f'{Api.api}/auth'
    user = f'{api}/user'
    token = f'{api}/token'
    create = f'{token}/create'

    @classmethod
    def delete(cls, userId: str):
        return f'{cls.user}/{userId}/token'

    @classmethod
    def metadata(cls, userId: str):
        return f'{cls.user}/{userId}/token'


class ApplicationViewModels(Enum):
    """
    [
        {
            "id": "string",
            "name": "string",
            "acronym": "string",
            "description": "string",
            "createdDate": "string",
            "createdByUser": {
                "id": "string",
                "name": "string"
            },
            "modifiedDate": "string",
            "modifiedByUser": {
                "id": "string",
                "name": "string"
            }
        }
    ]
    """
    id: str
    name: str
    acronym: str
    description: str
    createdDate: str
    createdByUser: {
        "id": str,
        "name": str
    }
    modifiedDate: str
    modifiedByUser: {
        "id": str,
        "name": str
    }


class Application(object):
    api = f'{Api.api}/app'
    light = f'{api}/light'

    @classmethod
    def by_id(cls, appId: str):
        return f'{cls.api}/{appId}'

    @classmethod
    def export(cls, appId: str):
        return f'{cls.api}/{appId}/export'


class Logging(object):
    api = f'{Api.api}/logging'
    job = f'{api}/job'
    recent = f'{api}/recent'

    @classmethod
    def by_id(cls, jobId: str):
        return f'{cls.job}/{jobId}'


class Record(object):

    @classmethod
    def api(cls, appId: str):
        return f'{Application.api}/{appId}/record'

    @classmethod
    def get(cls, appId: str, id: str):
        return f'{cls.api(appId)}/{id}'


class User(object):
    api = f'{Api.api}/user'
    login = f'{api}/login'
    authorize = f'{api}/authorize'


class Workspace(object):
    api = f'{Api.api}/workspaces'
    nav = f'{api}/nav'

    @classmethod
    def id(cls, id: str):
        """workspace specified by id"""
        return f'{cls.api}/{id}'

    @classmethod
    def app(cls, id: str):
        """workspaces for application id"""
        return f'{cls.api}/app/{id}'



from automon import log

from .app import app
from .config import FlaskConfig

logger = log.logging.getLogger(__name__)
logger.setLevel(level=log.DEBUG)


class FlaskClient(object):

    def __init__(
            self,
            host: str = None,
            port: int = None,
            debug: bool = True,
            config: FlaskConfig = None,
            **kwargs
    ):
        self.config = config or FlaskConfig(
            host=host,
            port=port,
            debug=debug,
            **kwargs
        )
        self.host = self.config.host
        self.port = self.config.port
        self.debug = self.config.debug

        self.app = app

    def run(self, **kwargs):
        self.app.run(
            host=self.host,
            port=self.port,
            debug=self.debug,
            **kwargs
        )



import unittest

from automon.integrations.keepass_to_pass import KeepassClient

test = KeepassClient()


class TestKeepassClient(unittest.TestCase):
    def test_set_database_csv_path(self):
        with self.assertRaises(FileNotFoundError):
            test.set_database_csv_path('AAAA.XXXCSV')

        open('BBBB.XXXCSV', 'w').close()

        self.assertTrue(test.set_database_csv_path('BBBB.XXXCSV'))

    def test_read_database_csv(self):
        with self.assertRaises(OSError):
            test.read_database_csv('AAAA.XXXCSV')


if __name__ == '__main__':
    unittest.main()



from automon.helpers.osWrapper import environ

from .attributes import LogRecordAttribute
from .logger import Logging, LogStream, TEST, DEBUG, INFO, WARN, ERROR, CRITICAL, NOTSET
from .logger import logging
from .logger import log_secret

log_format = f'{LogRecordAttribute(timestamp=True).levelname().name_and_lineno().funcName().message()}'
log_format_opentelemetry = environ('OTEL_PYTHON_LOG_FORMAT') or '\t'.join(
    [
        f'%(asctime)s',
        f'%(levelname)s',
        f'[%(name)s]',
        f'[%(filename)s:%(lineno)d]',
        f'[trace_id=%(otelTraceID)s span_id=%(otelSpanID)s resource.service.name=%(otelServiceName)s trace_sampled=%(otelTraceSampled)s]',
        f'%(funcName)s',
        f'%(message)s']
)

try:
    import opentelemetry
    from opentelemetry.instrumentation.logging import LoggingInstrumentor

    # logging.basicConfig(level=DEBUG, format=log_format_opentelemetry)
    LoggingInstrumentor().instrument(
        log_level=logging.DEBUG,
        set_logging_format=True,
        logging_format=log_format_opentelemetry)
except:
    logging.basicConfig(level=DEBUG, format=log_format)



import unittest
import asyncio

from automon.integrations.mac import MacChanger

client = MacChanger()


class MacChangerTest(unittest.TestCase):
    if asyncio.run(client.is_ready):
        def test_random_mac(self):
            self.assertTrue(asyncio.run(client.random_mac()))

        def test_ifconfig(self):
            self.assertTrue(asyncio.run(client.ifconfig()))
            self.assertTrue(asyncio.run(client.stdout()))
            self.assertFalse(asyncio.run(client.stderr()))


if __name__ == '__main__':
    unittest.main()



import os
import opentelemetry
from opentelemetry.trace import set_tracer_provider

from automon import environ
from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.logging.DEBUG)


class DatadogOpenTelemetryConfig(object):

    def __init__(self, host: str = 'https://api.datadoghq.com', api_key: str = None, app_key: str = None):
        self.host = host or environ('DD_SITE')
        self.api_key = api_key or environ('DD_API_KEY')
        self.app_key = app_key or environ('DD_APP_KEY')

        self.set_tracer_provider = False
        self.ddtrace_provider = None

    async def is_ready(self):
        if self.set_tracer_provider:
            return True
        logger.error(f'run {DatadogOpenTelemetryConfig.__name__}.{self.instrumentation.__name__}')

    async def instrumentation(self):
        # Must be set before ddtrace is imported!
        os.environ["DD_TRACE_OTEL_ENABLED"] = "true"

        import ddtrace.opentelemetry

        self.ddtrace_provider = ddtrace.opentelemetry.TracerProvider()
        opentelemetry.trace.set_tracer_provider(self.ddtrace_provider)
        self.set_tracer_provider = True

        return self.set_tracer_provider



import unittest

from automon.integrations.instagram.client_browser import InstagramBrowserClient

client = InstagramBrowserClient(headless=False)
client.start()


class InstagramClientTest(unittest.TestCase):

    def test(self):

        if client.is_ready():
            client.browser.get(client.urls.domain)
            if not client.is_authenticated():
                client.authenticate()
        client.browser.quit()


if __name__ == '__main__':
    unittest.main()



import unittest

from datetime import datetime

from automon import log
from automon.integrations.sentryio import SentryClient
from automon.integrations.geoip import Geoip

s = SentryClient()


class SentryClientTest(unittest.TestCase):
    def test_sentry(self):
        if s.isConnected():
            self.assertTrue(s.capture_exception(Exception(f'test capture_exception')))
            self.assertTrue(s.capture_message(f'test capture_message'))
            # self.assertTrue(s.capture_event('test capture_event', 'warning'))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults().enable_headless()


class SeleniumClientTest(unittest.TestCase):
    try:
        if browser.run():

            def test_logs(self):

                if browser.get('http://binance.com'):
                    logs = browser.get_log_performance()
                    logs = browser.get_log_browser()
                    logs = browser.get_log_driver()
                    logs = browser.get_logs()

                    browser.quit()

    except:
        pass


if __name__ == '__main__':
    unittest.main()



from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class MinioAssertions:

    @staticmethod
    def bucket_name(bucket: str) -> bool:
        if bucket == f'{bucket}'.lower():
            return f'{bucket}'
        logger.warning(msg=f'bucket name "{bucket}" must be lower')
        return f'{bucket}'.lower()



import unittest
import asyncio
import random

from automon.integrations.swimlaneWrapper.client import SwimlaneClientRest

client = SwimlaneClientRest()


class MyTestCase(unittest.TestCase):
    def test_login(self):
        if asyncio.run(client.is_ready()):
            if asyncio.run(client.login()):
                app_list = asyncio.run(
                    client.app_list())

                self.assertTrue(app_list)

                app = random.choice(app_list)
                appId = app['id']

                app_export = asyncio.run(
                    client.app_export(appId))

                self.assertTrue(app_export)

                app_get = asyncio.run(
                    client.app_by_id(appId=appId))

                self.assertTrue(app_get)

        pass


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.mac import os_is_mac
from automon.integrations.mac.wdutil import WdutilClient


class MyTestCase(unittest.TestCase):

    if os_is_mac():
        def test_something(self):
            self.assertTrue(WdutilClient().wdutil)


if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio

from automon.integrations.openTelemetryWrapper import OpenTelemetryConfig


class MyTestCase(unittest.TestCase):
    config = OpenTelemetryConfig()

    if asyncio.run(config.is_ready()):
        test = asyncio.run(config.test())

        def test_pop_finished_spans(self):
            spans = asyncio.run(self.config.pop_finished_spans())
            self.assertIsNotNone(spans)

            pass

    pass


if __name__ == '__main__':
    unittest.main()



import os


def environ(env_var: str, default: any = None) -> bool or str or None:
    """Get environment variable, else return default"""
    env = os.getenv(env_var)
    if env:
        if f'{env}'.lower() == 'true':
            return True
        if f'{env}'.lower() == 'false':
            return False
        return f'{env}'.strip()
    return default


def environ_list(env_var: str, delimiter: str = ',', default: list = []) -> list:
    """Get environment variable as comma-separated, else return default"""
    env = os.getenv(env_var)
    if env:
        env = env.split(delimiter)
        env = [str(x).strip() for x in env]
        return env
    return default



from automon import log
from automon.helpers import environ

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class SwimlaneConfig(object):

    def __init__(
            self,
            host: str = None,
            userName: str = '',
            password: str = '',
            apiKey: str = None,
            jwt_token: str = None,
    ):
        self.host = host or environ('SWIMLANE_HOST')
        self.userName = userName or environ('SWIMLANE_USERNAME', '')
        self.password = password or environ('SWIMLANE_PASSWORD', '')
        self.apiKey = apiKey or environ('SWIMLANE_APIKEY')
        self.jwt_token = jwt_token or environ('SWIMLANE_JWT_TOKEN', 'missing SWIMLANE_JWT_TOKEN')

        self.userName_model = None

        self.appId = environ('SWIMLANE_APP_ID')

    @property
    def access_token(self):
        """alias to private acces token"""
        return self.jwt_token

    @property
    def bearer_token(self):
        """token you get from username / password"""
        return self.token

    @property
    def credentials(self):
        if self.userName and self.password:
            return {
                'userName': self.userName,
                'password': self.password,
            }

    @property
    def token(self):
        return self.apiKey

    @property
    def headers(self):
        if self.token:
            return self.headers_api_token

        if self.private_token:
            return self.headers_jwt_token

    @property
    def headers_api_token(self):
        if self.token:
            return {
                'Authorization': f'Bearer {self.apiKey}'
            }

    @property
    def headers_jwt_token(self):
        if self.jwt_token:
            return {
                'Private-Token': self.jwt_token
            }

    async def is_ready(self) -> bool:
        if self.host:
            if self.userName and self.password:
                return True
            if self.apiKey:
                return True
            if self.jwt_token:
                return True

        logger.error(str(dict(
            host=self.host,
            userName=self.userName,
            password=self.password,
            apiKey=self.apiKey,
            jwt_token=self.jwt_token,
        )))
        return False

    @property
    def private_token(self):
        return self.jwt_token



import unittest

from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class LoggingTest(unittest.TestCase):

    def test_error(self):
        self.assertIsNone(logger.error('test'))

    def test_debug(self):
        self.assertIsNone(logger.debug('test'))

    def test_info(self):
        self.assertIsNone(logger.info('test'))

    def test_critical(self):
        self.assertIsNone(logger.critical('test'))

    def test_warn(self):
        self.assertIsNone(logger.warning('test'))


if __name__ == '__main__':
    unittest.main()



class XPaths(object):

    def __repr__(self):
        return 'Instagram XPaths'

    @property
    def authenticated_paths(self):
        authenticated = []
        authenticated.extend(self.profile_picture)
        authenticated.extend(self.home)
        return authenticated

    @property
    def home(self):
        return [
            '/html/body/div[2]/div/div/div[2]/div/div/div/div[1]/div[1]/div[1]/div/div/div/div/div[2]/div[1]/div/div/a/div',
        ]



import unittest
import asyncio
import sys

from automon.integrations.mac.airport import Airport

airport = Airport()


class AirportTest(unittest.TestCase):

    def test_run(self):
        if asyncio.run(airport.isReady()):
            self.assertTrue(asyncio.run(airport.run()))

    def test_scan(self):
        if asyncio.run(airport.isReady()):
            self.assertTrue(asyncio.run(airport.scan()))
            self.assertTrue(asyncio.run(airport.scan(0)))

    def test_summary(self):
        if asyncio.run(airport.isReady()):
            self.assertTrue(asyncio.run(airport.scan_summary()))
            self.assertTrue(asyncio.run(airport.scan_summary(0)))

    def test_xml(self):
        if asyncio.run(airport.isReady()):
            scan = asyncio.run(airport.scan_xml())
            if scan:
                self.assertTrue(scan)
            self.assertFalse(asyncio.run(airport.scan_xml(0)))

    def test_set_channel(self):
        if asyncio.run(airport.isReady()):
            pass
            # self.assertTrue(a.set_channel(10))

    def test_disassociate(self):
        if asyncio.run(airport.isReady()):
            pass
            # self.assertTrue(a.disassociate())

    def test_getinto(self):
        if asyncio.run(airport.isReady()):
            self.assertTrue(asyncio.run(airport.getinfo()))

    def test_create_psk(self):
        if asyncio.run(airport.isReady()):
            self.assertTrue(asyncio.run(airport.create_psk(ssid='AAAAAAAA', passphrase='CALVIN')))


if __name__ == '__main__':
    unittest.main()



import pandas

from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


def Series(*args, **kwargs) -> pandas.Series:
    s = pandas.Series(*args, **kwargs)
    logger.debug(s)
    return s



from automon import log
from automon.integrations.google.auth import GoogleAuthClient

from .config import GoogleSheetsConfig

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class Fields:
    hyperlink: str = 'sheets/data/rowData/values/hyperlink'


class ValueInputOption:
    USER_ENTERED: str = 'USER_ENTERED'
    RAW: str = 'RAW'


class GoogleSheetsClient(GoogleAuthClient):
    """Google Sheets client"""

    spreadsheetId: str
    worksheet: str
    range: str
    config: GoogleSheetsConfig

    def __init__(
            self,
            spreadsheetId: str = None,
            worksheet: str = '',
            range: str = 'A:Z',
            config: GoogleSheetsConfig = None,
            **kwargs
    ):
        super().__init__()
        self.config = config or GoogleSheetsConfig(
            spreadsheetId=spreadsheetId,
            **kwargs
        )

        self.worksheet = worksheet
        self.range = range

        self.response = None

    @property
    def values(self):
        """row values"""
        if self.response:
            try:
                return self.response['values']
            except Exception as error:
                pass

    def clear(
            self,
            range: str,
            spreadsheetId: str = None,
            **kwargs,
    ):
        """clear rows"""
        try:

            spreadsheets = self.spreadsheets()
            result = spreadsheets.values().clear(
                spreadsheetId=spreadsheetId or self.config.spreadsheetId,
                range=range or self.range,
                **kwargs,
            ).execute()

            logger.info(f'[google] :: sheets :: clear :: {result=}')
            logger.info(f"[google] :: sheets :: clear :: {result.get('clearedRange')} cells cleared.")
            return result
        except Exception as error:
            raise Exception(f'[google] :: sheets :: clear :: error :: {error}')

    def spreadsheets(self):
        """spreadsheet service"""
        service = self.service()
        return service.spreadsheets()

    def get(
            self,
            spreadsheetId: str = None,
            ranges: str = None,
            includeGridData: bool = False,
            fields: Fields or str = None,
            **kwargs,
    ):
        """get rows"""
        try:
            spreadsheets = self.spreadsheets()
            self.response = spreadsheets.get(
                spreadsheetId=spreadsheetId or self.config.spreadsheetId,
                ranges=ranges or self.range,
                includeGridData=includeGridData,
                fields=fields,
                **kwargs,
            ).execute()
            logger.info(f'[google] :: sheets :: get :: {self.worksheet}!{self.range} ({self.config.spreadsheetId=})')
        except Exception as error:
            raise Exception(f'[google] :: sheets :: get :: error {error=}')

        return self

    def get_values(
            self,
            spreadsheetId: str = None,
            range: str = None,
            **kwargs,
    ):
        """get values"""
        try:
            spreadsheets = self.spreadsheets()
            self.response = spreadsheets.values().get(
                spreadsheetId=spreadsheetId or self.config.spreadsheetId,
                range=range or f'{self.worksheet}!{self.range}',
                **kwargs,
            ).execute()

            logger.info(
                f'[google] :: '
                f'sheets :: '
                f'get values :: '
                f'{self.worksheet=} :: '
                f'{self.range=} :: '
                f'{self.config.spreadsheetId=}')
        except Exception as error:
            logger.error(f'[google] :: sheets :: get values :: {error=}')

        return self

    def list(self):
        # list(pageSize=1).execute()
        logger.warning(f'[google] :: sheets :: list :: {NotImplemented}')
        return

    def update(
            self,
            spreadsheetId: str = None,
            range: str = None,
            valueInputOption: ValueInputOption = ValueInputOption.USER_ENTERED,
            values: list = None,
    ):
        """update rows

        """
        try:

            body = {
                'values': values
            }

            logger.debug(f'[google] :: sheets :: update :: {body=}')

            spreadsheets = self.spreadsheets()
            result = spreadsheets.values().update(
                spreadsheetId=spreadsheetId or self.config.spreadsheetId,
                range=range or self.range,
                valueInputOption=valueInputOption,
                body=body
            ).execute()

            logger.info(f'[google] :: sheets :: update :: {result.get("updatedCells")} cells updated :: {result=}')
            return result
        except Exception as error:
            import traceback
            traceback.print_exc()
            raise Exception(f'[google] :: sheets :: update :: error :: {error}')



import json
import opentelemetry

from opentelemetry.util import types
from opentelemetry.trace import Status, StatusCode

from automon import log

from .config import OpenTelemetryConfig

logger = log.logging.getLogger(__name__)
logger.setLevel(log.logging.DEBUG)


class OpenTelemetryClient(object):

    def __init__(self):
        self.config = OpenTelemetryConfig()

    def add_event(self, name: str, attributes: types.Attributes = None, **kwargs):
        logger.debug(dict(name=name, attributes=attributes, kwargs=kwargs))
        return self.config.opentelemetry.trace.get_current_span.add_event(
            name=name,
            attributes=attributes,
            **kwargs
        )

    async def clear(self):
        logger.debug('clear')
        return await self.config.clear()

    async def is_ready(self):
        if await self.config.is_ready():
            return True

    async def get_finished_spans(self):
        logger.debug('get_finished_spans')
        return await self.config.get_finished_spans()

    async def pop_finished_spans(self):
        logger.debug('pop_finished_spans')
        return await self.config.pop_finished_spans()

    async def record_exception(self, exception: Exception):
        logger.error(f'{exception}')
        span = self.config.opentelemetry.trace.get_current_span()
        span.set_status(Status(StatusCode.ERROR))
        return span.record_exception(exception=exception)

    def start_as_current_span(
            self, name: str,
            attributes: any = None,
            **kwargs
    ):
        logger.debug(dict(name=name, attributes=attributes, kwargs=kwargs))
        return self.tracer.start_as_current_span(
            name=name,
            attributes=attributes,
            **kwargs,
        )

    async def start_consumer(self):
        """adds spans from memory to queue"""
        while True:
            pass
        return

    async def start_producer(self):
        """"""
        while True:
            pass
        return

    async def test(self):
        with self.tracer.start_as_current_span(name='rootSpan') as trace_root:
            trace_root.add_event('AAAAAAAA')

            with self.tracer.start_as_current_span(name='childSpan') as trace_child:
                trace_child.add_event('AAAAAAAA')
                trace_child.add_event('BBBBBBBB')

            trace_root.add_event('BBBBBBBB')

        return True

    async def to_dict(self):
        return [
            json.loads(span.to_json())
            for span in await self.get_finished_spans()
        ]

    async def to_datadog(self):
        log = []
        for span in await self.to_dict():
            message = dict(span).copy()
            ddsource = None
            ddtags = ','.join([f"{x[0]}:{x[1]}" for x in span.get("resource").get("attributes").items()])
            hostname = span['context']['trace_id']
            service = span['context']['span_id']

            log.append(dict(
                ddsource=ddsource,
                ddtags=ddtags,
                hostname=hostname,
                service=service,
                message=message,
            ))
        return log

    @property
    def tracer(self):
        return self.config.tracer



import unittest

from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults().enable_headless()


class SeleniumClientTest(unittest.TestCase):
    try:
        if browser.run():

            def test(self):

                if browser.get('http://1.1.1.1'):

                    if browser.check_page_load_finished():
                        self.assertTrue(browser.save_screenshot())
                        self.assertTrue(browser.save_screenshot(folder='./'))

                        browser.quit()

    except:
        pass


if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio

from automon.integrations.mac import MacChanger

client = MacChanger()


class MacChangerTest(unittest.TestCase):
    if asyncio.run(client.is_ready):
        def test_set_mac_random(self):
            # self.assertTrue(asyncio.run(client.set_mac_random()))
            # self.assertTrue(asyncio.run(client.stdout()))
            # self.assertFalse(asyncio.run(client.stderr()))
            pass


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults().enable_headless()


class SeleniumClientTest(unittest.TestCase):
    try:
        if browser.run():
            def test(self):
                self.assertTrue(browser.quit())

    except:
        pass


if __name__ == '__main__':
    unittest.main()



# import os
# import json
# import unittest
# import swimlane
#
# from swimlane import Swimlane
# from automon import environ
#
# appId = environ('SWIMLANE_APP_ID')
# host = environ('SWIMLANE_HOST')
# user = environ('SWIMLANE_USERNAME')
# password = environ('SWIMLANE_PASSWORD')
#
#
# class MyTestCase(unittest.TestCase):
#
#     if host and user and password and appId:
#         def test_login(self):
#             swimlane = Swimlane(host, user, password, verify_ssl=False)
#
#             app = swimlane.apps.get(id=appId)
#
#             # records = app.records.get()
#             # records = app.records.search('.')
#
#             # swimlane.exceptions.UnknownField: "<App: AUTO ASSET COLLECTION (AAC)> has no field 'test'"
#             record = app.records.create(
#                 json=json.dumps(dict(
#                     string='string',
#                     int=1,
#                     list=[1, 2, 3],
#                     dict=dict(
#                         key='value'
#                     )
#                 ))
#             )
#
#             pass
#
#
# if __name__ == '__main__':
#     unittest.main()



import asyncio
import unittest

from automon.integrations.datadogWrapper import DatadogClientRest
from automon.integrations.openTelemetryWrapper import OpenTelemetryClient


class MyTestCase(unittest.TestCase):
    client = DatadogClientRest()
    client_tracer = OpenTelemetryClient()

    def test_log(self):
        if asyncio.run(self.client.is_ready()):

            asyncio.run(self.client_tracer.config.test())

            spans = asyncio.run(self.client_tracer.to_datadog())
            for span in spans:
                asyncio.run(
                    self.client.log(
                        ddsource=span['ddsource'],
                        ddtags=span['ddtags'],
                        hostname=span['hostname'],
                        service=span['service'],
                        message=span['message'],
                    ))

            pass

    if __name__ == '__main__':
        unittest.main()



import requests

from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.INFO)


class RequestsConfig(object):

    def __init__(self):
        pass

    def __repr__(self):
        return f'{NotImplemented}'

    @property
    def is_ready(self):
        return f'{NotImplemented}'



import asyncio
import unittest

from automon.integrations.requestsWrapper.rest import BaseRestClient


class Inherit(BaseRestClient):

    def __init__(self):
        super().__init__()
        pass


class Client(unittest.TestCase):
    def test_get(self):
        self.assertTrue(asyncio.run(Inherit().get(url='https://1.1.1.1')))


if __name__ == '__main__':
    unittest.main()



import pandas

from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


def DataFrame(*args, **kwargs) -> pandas.DataFrame:
    df = pandas.DataFrame(*args, **kwargs)
    logger.debug(df)
    return df



import unittest

from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

browser = SeleniumBrowser()
browser.config.webdriver_wrapper = ChromeWrapper()
browser.config.webdriver_wrapper.enable_defaults().enable_headless()

agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:112.0) Gecko/20100101 Firefox/112.0'

browser.config.webdriver_wrapper.set_user_agent(agent)


class SeleniumClientTest(unittest.TestCase):
    try:
        if browser.run():
            def test_user_agent(self):
                self.assertEqual(browser.user_agent, agent)

                browser.quit()
    except:
        pass


if __name__ == '__main__':
    unittest.main()



from automon import log

from .config_opentelemetry import DatadogOpenTelemetryConfig

logger = log.logging.getLogger(__name__)
logger.setLevel(log.logging.DEBUG)


class DatadogOpenTelemetryClient(object):

    def __init__(self, host: str = None, api_key: str = None, app_key: str = None):
        self.config = DatadogOpenTelemetryConfig(host=host, api_key=api_key, app_key=app_key)

    async def is_ready(self):
        if await self.config.is_ready():
            return True
        logger.error(f'client not ready')



from automon import Run
from automon import log
from automon import environ
from automon.integrations.mac import os_is_mac

from .exceptions import *

logger = log.logging.getLogger(__name__)
logger.setLevel(level=log.DEBUG)


class WdutilConfig(object):

    def __init__(self, password: str = None, wdutil_path: str = None):
        self.password = password or environ('WDUTIL_PASSWORD')
        self._wdutil_path = wdutil_path
        self._runner = Run()

    def is_ready(self):
        if not self.password:
            logger.error(f'missing WDUTIL_PASSWORD')

        if not self.wdutil_path():
            logger.error(f'missing wdutil')

        if self.password and self.wdutil_path():
            return True

        return False

    def wdutil_path(self):
        if os_is_mac():

            if self._wdutil_path:
                return self._wdutil_path

            if self._runner.which('wdutil'):
                self._wdutil_path = self._runner.stdout.decode().strip()
                logger.info(str(dict(
                    wdutil_path=self._wdutil_path
                )))
                return self._wdutil_path

        return False



# import unittest
#
# from ..client import FlaskClient
#
#
# class TestFlask(unittest.TestCase):
#     test = FlaskClient()
#
#     def test_run(self):
#         self.test.run()
#
#
# if __name__ == '__main__':
#     unittest.main()



from automon import log
from automon.helpers import Run

logger = log.logging.getLogger(__name__)
logger.setLevel(log.ERROR)


class NmapConfig(object):
    def __init__(self, **kwargs):
        self.nmap = None

    def __repr__(self):
        if self.isReady():
            return f'{self.nmap}'
        return f'nmap not found'

    def isReady(self, **kwargs):
        check = Run('which nmap', **kwargs)

        if check.stdout:
            self.nmap = check.stdout.decode().strip()
            logger.debug(f'nmap located, {self.nmap}')
            return True
        else:
            logger.error(f'nmap not found')

        return False



import unittest
import asyncio

from automon.integrations.openTelemetryWrapper import OpenTelemetryConfig


class MyTestCase(unittest.TestCase):
    config = OpenTelemetryConfig()

    def test_something(self):
        self.assertTrue(asyncio.run(
            self.config.test())
        )


if __name__ == '__main__':
    unittest.main()



import unittest
import asyncio
import random

from automon.integrations.swimlaneWrapper.client import SwimlaneClientRest

client = SwimlaneClientRest()


class MyTestCase(unittest.TestCase):
    def test_login(self):
        if asyncio.run(client.is_ready()):
            if asyncio.run(client.login_token()):
                logs = asyncio.run(
                    client.logging_recent())

                jobId = '6602057d6158e302aff869b2'

                logs_job = asyncio.run(
                    client.logging_by_id(jobId=jobId))


        pass


if __name__ == '__main__':
    unittest.main()



import asyncio
import unittest

from automon.integrations.datadogWrapper import DatadogClientRest


class MyTestCase(unittest.TestCase):
    client = DatadogClientRest()

    if asyncio.run(client.is_ready()):
        def test_auth(self):
            self.assertTrue(asyncio.run(
                self.client.validate()
            ))

        pass


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.xsoar import XSOARConfig


class MyTestCase(unittest.TestCase):
    test = XSOARConfig()

    if test.is_ready():
        def test_config(self):
            self.assertTrue(self.test.is_ready())


if __name__ == '__main__':
    unittest.main()



import re


def geolocation(string):
    """Parse any kind of geolocation data"""

    # TODO: parse any geolocation info (long, lat)

    pattern = [
        r'([Long]{4}:[ ]?[0-9\.]*,[ ]?[Lat]{3}:[ ]?[0-9\.]*)'
    ]

    for p in pattern:
        c = re.compile(p)
        r = re.findall(c, string)

        if r:
            return r



import automon
import warnings
import functools
import googleapiclient.http
import googleapiclient.discovery
import google.auth.transport.requests

from .config import GoogleAuthConfig

logger = automon.log.logging.getLogger(__name__)
logger.setLevel(automon.log.DEBUG)


class GoogleAuthClient(object):
    """Google Auth client"""

    def __init__(
            self,
            config: GoogleAuthConfig = None,
            serviceName: str = None,
            scopes: list = None,
            version: str = None,
            **kwargs,
    ):

        self.config = config or GoogleAuthConfig(
            serviceName=serviceName,
            scopes=scopes,
            version=version,
            **kwargs
        )

    def __repr__(self):
        return f'{self.__dict__}'

    @classmethod
    def execute(cls, func):
        return func.execute()

    def _is_connected(func):
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            if self.authenticate():
                return func(self, *args, **kwargs)

        return wrapped

    def authenticate(self) -> bool:
        """authenticate with credentials"""

        try:
            if self.authenticate_oauth():
                return self.authenticate_oauth()
        except Exception as error:
            raise error

        try:
            if self.authenticate_service_account():
                return self.authenticate_service_account()
        except Exception as error:
            raise error

        return False

    def authenticate_oauth(self) -> bool:
        """authenticate web token"""

        creds = self.config.Credentials()

        if hasattr(creds, 'refresh_token'):
            try:
                creds.refresh(google.auth.transport.requests.Request())
                logger.info(f'[google] :: auth :: oauth :: token refresh :: {getattr(creds, "refresh_token")}')
                logger.info(f'[google] :: auth :: oauth :: token refresh :: done')
                return True
            except Exception as error:
                logger.error(msg=f'[google] :: auth :: oauth :: error :: token refresh failed: {error}')

        else:
            warnings.warn(f'TODO: add google flow() authentication')
            logger.info(f'[google] :: auth :: oauth :: done')
            return True

        return False

    def authenticate_service_account(self) -> bool:
        """authenticate service account"""
        if self.config.Credentials():
            return True
        return False

    def is_connected(self) -> bool:
        """Check if authenticated to make requests"""
        return self.authenticate()

    def service(
            self,
            serviceName: str = None,
            version: str = None,
            http=None,
            discoveryServiceUrl=None,
            developerKey=None,
            model=None,
            requestBuilder=None,
            credentials=None,
            cache_discovery=True,
            cache=None,
            client_options=None,
            adc_cert_path=None,
            adc_key_path=None,
            num_retries=1,
            static_discovery=None,
            always_use_jwt_access=False,
            **kwargs
    ) -> googleapiclient.discovery.build:
        return googleapiclient.discovery.build(
            serviceName=serviceName or self.config.serviceName,
            version=version or self.config.version,
            http=http,
            discoveryServiceUrl=discoveryServiceUrl,
            developerKey=developerKey,
            model=model,
            requestBuilder=requestBuilder or googleapiclient.http.HttpRequest,
            credentials=credentials or self.config.Credentials(),
            cache_discovery=cache_discovery,
            cache=cache,
            client_options=client_options,
            adc_cert_path=adc_cert_path,
            adc_key_path=adc_key_path,
            num_retries=num_retries,
            static_discovery=static_discovery,
            always_use_jwt_access=always_use_jwt_access,
            **kwargs,
        )



from automon import log
from automon import log_secret
from automon.helpers import Run

from .config import WdutilConfig

logger = log.logging.getLogger(__name__)
logger.setLevel(level=log.DEBUG)


class WdutilClient(object):

    def __init__(self, config: WdutilConfig = None, wdutil_path: str = None):
        self.config = config or WdutilConfig(wdutil_path=wdutil_path)
        self.wdutil = self.config.wdutil_path()

        self._runner = Run()

        self.output = None

    def info(self):
        return self.run('info')

    def run(self, arg: str):
        self.config.is_ready()

        secret = f'echo {self.config.password} | '
        command = f'sudo -S {self.wdutil} {arg}'

        logger.info(f'echo {log_secret(self.config.password)} | {command}')
        run = self._runner.run(command=f'{secret}{command}', shell=True)

        output = self._runner.stdout_lines
        self.output = WdutilOutput(output=output)

        return run

    def is_ready(self):
        if self.config.is_ready():
            return True
        return False

    def help(self):
        return self.run('help')


class WdutilOutput(object):
    NETWORK: None
    WIFI: None
    BLUETOOTH: None
    AWDL: None

    def __init__(self, output: [str]):
        self._output = output

    @property
    def network(self):
        pass

    @property
    def wifi(self):
        pass

    @property
    def bluetooth(self):
        pass

    @property
    def awdl(self):
        pass



import unittest
import asyncio

from automon.integrations.swimlaneWrapper.client import SwimlaneClientRest

client = SwimlaneClientRest()


class MyTestCase(unittest.TestCase):
    def test_login(self):
        if asyncio.run(client.is_ready()):
            if asyncio.run(client.login()):
                app_id = client.config.appId
                fields = asyncio.run(
                    client.record_resolve_fields(appId=app_id)
                )

                self.assertTrue(fields)

        pass


if __name__ == '__main__':
    unittest.main()



import os
import warnings
import pandas as pd

from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.ERROR)


class Grok:
    def __init__(self):

        p = 'logstash-patterns-core/patterns'
        l = f'{os.path.join(os.path.split(os.path.realpath(__file__))[0])}'

        walk = os.walk(l)

        for folder, folders, files in walk:
            f_list = [os.path.join(folder, f) for f in files]

            if 'legacy' in folder:
                self.legacy_df = self.to_df(f_list)
                self.legacy = self.legacy_df.to_dict()[0]

            if 'ecs-v1' in folder:
                self.ecs_v1_df = self.to_df(f_list)
                self.ecs_v1 = self.ecs_v1_df.to_dict()[0]

        pass

    def to_df(self, files: list) -> pd.DataFrame:

        patterns = [self.to_pattern(f) for f in files]
        return self.expanded_dict(patterns)

    def to_pattern(self, file: str) -> pd.Series:
        filename = os.path.split(file)[-1]
        pattern = open(file, 'r').read()
        pattern = [x for x in pattern.splitlines() if '#' not in x if x]
        pattern = [x.split(' ') for x in pattern]
        pattern = {k[0]: ''.join(k[1:]) for k in pattern}

        return pd.Series(pattern).rename(filename)

    def expanded_dict(self, patterns: list) -> pd.DataFrame:

        # big_dict = {}
        # for d in patterns:
        #     for k, v in d.items():
        #         if k not in big_dict.keys():
        #             big_dict[k] = v
        #         else:
        #             logger.info(f'skipping existing, {k}')

        df = pd.DataFrame(pd.concat(patterns))
        return df



import sys
import random

from automon import log, Run

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class MacChanger(object):

    def __init__(self):
        self.mac_fixed = None
        self.mac_new = None

        self._run = Run()

    async def args_expansion(self, args: tuple) -> str:
        return ' '.join(args)

    async def ifconfig(self, *args, **kwargs):
        args = await self.args_expansion(args)
        return self._run.run(f'ifconfig {args}', **kwargs)

    @property
    async def is_ready(self):
        logger.debug(sys.platform)
        if sys.platform == 'darwin':
            return True

    async def random_mac(self) -> str:
        """return random mac address

        Example:
            return 3c:a6:f6:16:da:66
        """
        mac = []

        while len(mac) < 6:
            mac.append(
                random.choice(range(0, 255))
            )

        mac = [hex(x) for x in mac]
        mac = [str(x).split('x') for x in mac]
        mac = [list(x)[1] for x in mac]
        mac = ':'.join(mac)

        logger.debug(mac)

        return mac

    async def set_mac(self, mac: str, **kwargs):
        return await self.ifconfig('en0', 'link', mac, **kwargs)

    async def set_mac_random(self, **kwargs):
        return await self.set_mac(mac=await self.random_mac(), **kwargs)

    async def stdout(self):
        logger.debug(self._run.stdout)
        return self._run.stdout

    async def stderr(self):
        logger.debug(self._run.stderr)
        return self._run.stderr

    async def sudo_ifconfig(self, *args, stdin: str = None, **kwargs) -> bool:
        args = await self.args_expansion(args)
        return self._run.run(f'sudo -S ifconfig {args}', shell=True, stdin=stdin, **kwargs)



import asyncio
import opentelemetry

from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import SimpleSpanProcessor
from opentelemetry.sdk.trace.export.in_memory_span_exporter import InMemorySpanExporter

from automon import log

logger = log.logging.getLogger(__name__)
logger.setLevel(log.logging.DEBUG)


class OpenTelemetryConfig(object):
    def __init__(self):
        self.opentelemetry = opentelemetry
        self.provider = TracerProvider()
        self.memory_processor = InMemorySpanExporter()
        self.processor = SimpleSpanProcessor(self.memory_processor)
        self.provider.add_span_processor(self.processor)

        opentelemetry.trace.set_tracer_provider(self.provider)

        self.tracer = opentelemetry.trace.get_tracer(__name__)

        self.queue_consumer = asyncio.Queue()
        self.queue_producer = asyncio.Queue()

    async def clear(self):
        logger.debug('clear')
        return self.memory_processor.clear()

    @property
    def get_current_span(self):
        logger.debug('get_current_span')
        return opentelemetry.trace.get_current_span()

    async def is_ready(self):
        if self.provider and self.memory_processor and self.processor:
            return True

    async def get_finished_spans(self):
        logger.debug('get_finished_spans')
        return self.memory_processor.get_finished_spans()

    async def pop_finished_spans(self):
        """ideal is to lock, pop spans, and clear"""
        spans = await self.get_finished_spans()
        clear = await self.clear()
        return spans

    async def test(self):
        with self.tracer.start_as_current_span(name="rootSpan") as trace_root:
            trace_root.add_event('AAAAAAAA')
            with self.tracer.start_as_current_span(name="childSpan") as trace_child:
                trace_child.add_event('AAAAAAAA')
                trace_child.add_event('BBBBBBBB')

        return True



import json
import requests

from automon import log
from .config import RequestsConfig

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class RequestsClient(object):
    def __init__(self, url: str = None, data: dict = None, headers: dict = None,
                 config: RequestsConfig = None):
        """Wrapper for requests library"""

        self.config = config or RequestsConfig()

        self.url: str = url
        self.data: dict = data
        self.errors: bytes = b''
        self.headers: dict = headers
        self.response = None
        self.requests = requests
        self.session = self.requests.Session()

    def __repr__(self):
        return f'{self.__dict__}'

    def __len__(self):
        if self.content:
            len(self.content)

    async def _log_result(self):
        if self.status_code == 200:
            msg = [
                self.response.request.method,
                self.response.url,
                f'{round(len(self.content) / 1024, 2)} KB',
                f'{self.status_code}',
            ]
            msg = ' '.join(msg)
            return logger.debug(msg)

        msg = [
            self.response.request.method,
            self.response.url,
            f'{round(len(self.content) / 1024, 2)} KB',
            f'{self.status_code}',
            f'{self.content}'
        ]

        msg = ' '.join(msg)
        return logger.error(msg)

    async def _params(self, url, data, headers):
        if url is None:
            url = self.url

        if data is None:
            data = self.data

        if headers is None:
            headers = self.headers

        self.url = url
        self.data = data
        self.headers = headers
        return url, data, headers

    @property
    def content(self):
        if 'content' in dir(self.response):
            return self.response.content

    async def content_to_dict(self):
        return await self.to_dict()

    async def delete(
            self,
            url: str = None,
            data: dict = None,
            headers: dict = None,
            **kwargs
    ) -> bool:
        """requests.delete"""

        url, data, headers = await self._params(url, data, headers)

        try:
            self.response = self.session.delete(url=url, data=data, headers=headers, **kwargs)
            await self._log_result()

            if self.status_code == 200:
                return True

            return False
        except Exception as e:
            self.errors = e
            logger.error(f'delete failed. {e}')
        return False

    async def get(
            self,
            url: str = None,
            data: dict = None,
            headers: dict = None,
            **kwargs
    ) -> bool:
        """requests.get"""

        url, data, headers = await self._params(url, data, headers)

        try:
            self.response = self.session.get(url=url, data=data, headers=headers, **kwargs)

            logger.debug(
                f'{self.response.url} '
                f'{round(len(self.content) / 1024, 2)} KB '
                f'{self.status_code}'
            )

            if self.status_code == 200:
                return True

            self.errors = self.content

            return False
        except Exception as e:
            self.errors = e
            logger.error(f'{e}')
        return False

    async def patch(
            self,
            url: str = None,
            data: dict = None,
            headers: dict = None,
            **kwargs
    ) -> bool:
        """requests.patch"""

        url, data, headers = await self._params(url, data, headers)

        try:
            self.response = self.session.patch(url=url, data=data, headers=headers, **kwargs)

            logger.debug(
                f'{self.response.url} '
                f'{round(len(self.content) / 1024, 2)} KB '
                f'{self.status_code}'
            )

            if self.status_code == 200:
                return True

            self.errors = self.content

            return False
        except Exception as e:
            self.errors = e
            logger.error(f'patch failed. {e}')
        return False

    async def post(
            self,
            url: str = None,
            data: dict = None,
            headers: dict = None,
            **kwargs
    ) -> bool:
        """requests.post"""

        url, data, headers = await self._params(url, data, headers)

        try:
            self.response = self.session.post(url=url, data=data, headers=headers, **kwargs)

            logger.debug(
                f'{self.response.url} '
                f'{round(len(self.content) / 1024, 2)} KB '
                f'{self.status_code}'
            )

            if self.status_code == 200:
                return True

            self.errors = self.content

            return False
        except Exception as e:
            self.errors = e
            logger.error(f'post failed. {e}')
        return False

    async def put(
            self,
            url: str = None,
            data: dict = None,
            headers: dict = None,
            **kwargs
    ) -> bool:
        """requests.put"""

        url, data, headers = await self._params(url, data, headers)

        try:
            self.response = self.session.put(url=url, data=data, headers=headers, **kwargs)

            logger.debug(
                f'{self.response.url} '
                f'{round(len(self.content) / 1024, 2)} KB '
                f'{self.status_code}'
            )

            if self.status_code == 200:
                return True

            self.errors = self.content

            return False
        except Exception as e:
            self.errors = e
            logger.error(f'put failed. {e}')
        return False

    @property
    def reason(self):
        if 'reason' in dir(self.response):
            return self.response.reason

    @property
    def status_code(self):
        if 'status_code' in dir(self.response):
            return self.response.status_code

    @property
    def text(self):
        if self.response:
            return self.response.text

    async def to_dict(self):
        if self.response is not None:
            try:
                return json.loads(self.content)
            except Exception as error:
                logger.error(error)

    async def to_json(self):
        if self.content:
            try:
                return json.dumps(json.loads(self.content))
            except Exception as error:
                logger.error(error)

    async def update_headers(self, headers: dict):
        return self.session.headers.update(headers)


class Requests(RequestsClient):
    pass



import unittest
import asyncio

from automon.integrations.swimlaneWrapper.client import SwimlaneClientRest

client = SwimlaneClientRest()


class MyTestCase(unittest.TestCase):
    def test_login(self):
        if asyncio.run(client.is_ready()):
            self.assertTrue(
                asyncio.run(client.login_username_password())
            )
            self.assertFalse(
                asyncio.run(client.login_token())
            )
            self.assertFalse(
                asyncio.run(client.create_auth_token())
            )


if __name__ == '__main__':
    unittest.main()



import subprocess

from pprint import pprint

from automon import log
from automon.helpers.dates import Dates

from .exceptions import *

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class Run:
    command: str

    def __init__(self, command: str = None, **kwargs):
        """Run shell"""

        self.last_run = None
        self.command = command

        self._stdout = b''
        self._stderr = b''

        self.call = None
        self.returncode = None

        if command:
            self.run(command=command, **kwargs)

    def rc(self):
        if self.call:
            self.returncode = self.call.returncode
            return self.returncode

    def Popen(self, *args, **kwargs):
        return self.run(*args, **kwargs)

    def pretty(self):
        template = f"""stdout:
{self.stdout.decode()}
stderr:
{self.stderr.decode()}"""
        pprint(template)

    def print(self):
        template = f"""stdout:
{self.stdout.decode()}

stderr:
{self.stderr.decode()}"""
        print(template)

    def set_command(self, command: str) -> bool:
        logger.debug(command)
        if command:
            self.command = command
            return True
        return False

    @property
    def stdout(self) -> bytes:
        if type(self._stdout) is str:
            return str(self._stdout).encode()

        return self._stdout

    @property
    def stdout_lines(self):
        return self.stdout.decode().splitlines()

    @property
    def stderr(self) -> bytes:
        if type(self._stderr) is str:
            return str(self._stderr).encode()

        return self._stderr

    @property
    def stderr_lines(self):
        return self.stderr.decode().splitlines()

    def which(self, program: str, *args, **kwargs) -> bool:
        """runs which

        :param program:
        :return:
        """
        logger.debug(str(dict(
            which=program,
            args=args,
            kwargs=kwargs,
        )))
        return self.run(command=f'which {program}', *args, **kwargs)

    def run_command(self, *args, **kwargs) -> bool:
        """alias to run"""
        logger.debug(str(dict(
            args=args,
            kwargs=kwargs,
        )))
        return self.run(*args, **kwargs)

    def run(
            self,
            command: str,
            pipe: bool = False,
            text: bool = False,
            shell: bool = False,
            sanitize_command: bool = True,
            **kwargs
    ) -> bool:

        if not pipe:
            self.command = command

        if sanitize_command and not shell:
            command = self.sanitize_command(command)

        if not command:
            logger.error(str(dict(
                command=command,
                text=text,
                shell=shell,
                sanitize_command=sanitize_command,
                kwargs=kwargs,
            )))
            raise SyntaxError(f'command cannot be empty, {command}')

        logger.debug(str(dict(
            command=command,
            text=text,
            shell=shell,
            sanitize_command=sanitize_command,
            kwargs=kwargs,
        )))

        try:

            self.call = subprocess.Popen(
                args=command,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=text,
                shell=shell,
                **kwargs)

            stdout, stderr = self.call.communicate()
            # self.call.wait()

            timestamp = Dates.iso()

            self.last_run = timestamp
            self._stdout = stdout
            self._stderr = stderr
            self.returncode = self.call.returncode

            if self.returncode == 0:
                logger.debug(str(dict(
                    stdout_KB=round(len(self.stdout) / 1024, 2),
                    stderr_KB=round(len(self.stderr) / 1024, 2),
                )))
                return True

        except Exception as error:
            self._stderr = f'{error}'.encode()
            logger.error(f'{error}')
            raise RuntimeError(error)

        logger.error(str(dict(
            stdout_KB=round(len(self.stdout) / 1024, 2),
            stderr_KB=round(len(self.stderr) / 1024, 2),
        )))
        return False

    def sanitize_command(self, command: str) -> [str]:

        if isinstance(command, str):

            if '|' in command:
                command = self.sanitize_command_pipe(command=command)
                command = [self.sanitize_command_spaces(command=cmd) for cmd in command]
            else:
                command = self.sanitize_command_spaces(command=command)

        logger.debug(str(dict(
            command=command
        )))
        return command

    def sanitize_command_pipe(self, command: str) -> [str]:
        """support for shell command piping"""
        error = f'Pipes are not supported! To use run(shell=True). {command}'
        logger.error(error)
        raise NotSupportedCommand(error)

        split_command = f'{command}'.split('|')
        return split_command

    def sanitize_command_spaces(self, command: str) -> [str]:
        split_command = f'{command}'.split(' ')
        split_command = [str(x).strip() for x in split_command]
        split_command = [x for x in split_command if x]
        return split_command

    def __repr__(self) -> str:
        return str(dict(
            command=self.command,
            stdout=f'{round(len(self.stdout) / 1024, 2)} KB',
            stderr=f'{round(len(self.stderr) / 1024, 2)} KB',
        ))

    def __len__(self):
        return sum([len(self.stdout), len(self.stderr)])

    def __eq__(self, other):
        if isinstance(other, Run):
            self.command == other.command
            return True
        return False



import os

from automon import log
from automon.helpers.sanitation import Sanitation
from automon.helpers.osWrapper.environ import environ

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class Neo4jConfig:
    def __init__(
            self, user: str = None,
            password: str = None,
            hosts: str = None,
            encrypted: bool = None,
            trust: bool = None):
        """Neo4j config
        """

        self.NEO4J_USER = user or environ('NEO4J_USER', '')
        self.NEO4J_PASSWORD = password or environ('NEO4J_PASSWORD', '')
        self.NEO4J_HOST = hosts or environ('NEO4J_HOST', '')

        self.encrypted = encrypted
        self.trust = trust

    @property
    def is_ready(self) -> bool:
        if self.NEO4J_USER and self.NEO4J_PASSWORD and self.NEO4J_HOST:
            return True
        if not self.NEO4J_USER:
            logger.error(f'missing NEO4J_USER')
        if not self.NEO4J_PASSWORD:
            logger.error(f'missing NEO4J_PASSWORD')
        if not self.NEO4J_HOST:
            logger.error(f'missing NEO4J_HOST')
        return False

    def __repr__(self):
        return f'{self.__dict__}'



import unittest
import asyncio

from automon.helpers.sleeper import Sleeper


class SleeperTest(unittest.TestCase):
    def test_Sleeper(self):
        loop = asyncio.new_event_loop()
        task = loop.run_until_complete(Sleeper.seconds_async(1))


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.log import log_secret


class MyTestCase(unittest.TestCase):
    def test_something(self):
        self.assertEqual(log_secret(secret='password'), '********************************')  # add assertion here


if __name__ == '__main__':
    unittest.main()



import json

from automon import log
from automon.integrations.requestsWrapper import RequestsClient

from .config import SwimlaneConfig
from .api.v2 import *

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class SwimlaneClient(object):
    pass


class SwimlaneClientRest(object):

    def __init__(self):
        self.config = SwimlaneConfig()
        self.requests = RequestsClient()

        self.auth = None
        self.apps = None
        self.records = None
        self.workspaces = None

    async def is_ready(self):
        if await self.config.is_ready():
            if self.config.headers:
                return True

    async def test_connection(self):
        return

    async def login(self):
        """tries all types of login"""

        if await self.login_username_password():
            return True

        if await self.login_token():
            return True

        return False

    async def login_username_password(self) -> bool:
        """Login with username and password"""
        url = f'{self.host}/{User.login}'

        response = await self.requests.post(
            url=url,
            json=self.config.credentials,
        )

        apiKey = dict(json.loads(self.requests.content)).get('token')
        self.config.apiKey = apiKey

        self.requests.session.headers.update(self.config.headers)

        self.config.userName_model = await self.requests.to_dict()

        return response

    async def login_token(self) -> bool:
        """Login with username and password"""
        url = f'{self.host}/{User.authorize}'

        self.requests.session.headers.update(self.config.headers_jwt_token)

        response = await self.requests.get(
            url=url,
        )

        self.config.userName_model = await self.requests.to_dict()

        return response

    async def logging_by_id(self, jobId: str):
        """finds the recent logs"""
        url = f'{self.host}/{Logging.by_id(jobId=jobId)}'

        response = await self.requests.get(url=url)
        logs_job = await self.requests.to_dict()

        return logs_job

    async def logging_recent(self, level: str = 'Debug'):
        """gets logs but needs to get it from a task run id

        this is broken
        """
        url = f'{self.host}/{Logging.recent}'

        request_body = {
            "level": [
                level
            ]
        }

        header = {
            'Content-Type': 'application/json'
        }

        # self.requests.session.headers.update(header)

        response = await self.requests.post(url=url, json=request_body, headers=header)
        logs_recent = await self.requests.to_dict()

        return logs_recent

    async def create_auth_token(self):
        """Creates a new access token for the user making the request"""
        url = f'{self.host}/{Auth.create}'

        response = await self.requests.post(
            url=url,
        )

        return response

    async def app_list(self):
        url = f'{self.host}/{Application.api}'

        response = await self.requests.get(
            url=url,
        )

        self.apps = await self.requests.to_dict()

        return self.apps

    async def app_by_id(self, appId: str):
        url = f'{self.host}/{Application.by_id(appId=appId)}'

        response = await self.requests.get(
            url=url,
        )

        app = await self.requests.to_dict()

        return app

    async def app_export(self, appId: str):
        url = f'{self.host}/{Application.export(appId=appId)}'

        response = await self.requests.get(
            url=url,
        )

        app = await self.requests.to_dict()

        return app

    @property
    def host(self):
        return self.config.host

    async def record_resolve_fields(self, appId: str):
        """since swimlane has no documentation on resolving field names"""

        url = f'{self.host}/{Record.api(appId)}'

        response = await self.requests.get(
            url=url,
        )

        record_hashmap = {}

        record = await self.requests.to_dict()
        record_values = dict(record.get('values'))

        for item in record_values.items():
            key, value = item
            if '$' in key:
                continue
            record_hashmap[key] = key

        logger.debug(record_hashmap)

        record = {
            "applicationId": appId,
            "values": {
                "$type": "System.Collections.Generic.Dictionary`2[[System.String, mscorlib],[System.Object, mscorlib]], mscorlib",
            }
        }

        record.get('values').update(record_hashmap)

        record_create = await self.record_create_hard(appId=appId, data=record)

        return record_hashmap

    async def record_schema(self, appId: str):
        url = f'{self.host}/{Record.api(appId)}'

        response = await self.requests.get(
            url=url,
        )

        record = await self.requests.to_dict()

        return record

    async def record_create(self, appId: str, key: str, value: str or int):
        """create a record"""
        return await self.record_create_easy(appId=appId, key=key, value=value)

    async def record_create_easy(self, appId: str, key: str, value: str or int):
        """create a record with boilerplate added

        The bare minimum you need to send is (assuming application id is 5667113fd273a205bc747cf0):
        {
          "applicationId": "5667113fd273a205bc747cf0",
          "values": {
            "$type": "System.Collections.Generic.Dictionary`2[[System.String, mscorlib],[System.Object, mscorlib]], mscorlib",
            "56674c5cc6c7dea0aeab4aed": "A new value"
          }
        }

        """
        url = f'{self.host}/{Record.api(appId)}'

        record = {
            "applicationId": appId,
            "values": {
                "$type": "System.Collections.Generic.Dictionary`2[[System.String, mscorlib],[System.Object, mscorlib]], mscorlib",
                key: value
            }
        }

        record_json = json.dumps(record)

        response = await self.requests.post(
            url=url,
            json=record
        )

        record_created = await self.requests.content_to_dict()

        return record_created

    async def record_create_hard(self, appId: str, data: dict):
        """create a record the hard way

        no handholding. you're on your own"""
        url = f'{self.host}/{Record.api(appId)}'

        response = await self.requests.post(
            url=url,
            json=data
        )

        record_created = await self.requests.content_to_dict()

        return record_created

    async def record_delete_all(self, appId: str):
        """delete all records in application"""
        url = f'{self.host}/{Record.api(appId)}'

        response = await self.requests.delete(
            url=url
        )

        if self.requests.status_code == 204 or self.requests.status_code == 404:
            return True

        return response

    async def record_get(self, appId: str, id: str):
        """get a record"""
        url = f'{self.host}/{Record.get(appId=appId, id=id)}'

        response = await self.requests.get(
            url=url
        )

        record = await self.requests.to_dict()

        return record

    async def record_get_base(self, appId: str):
        """get a record"""
        url = f'{self.host}/{Record.api(appId=appId)}'

        response = await self.requests.get(
            url=url
        )

        return response

    @property
    def userId(self):
        return self.config.userName

    async def workspace_list(self):
        url = f'{self.host}/{Workspace.api}'

        response = await self.requests.get(
            url=url,
        )

        self.workspaces = await self.requests.to_dict()

        return self.workspaces



import unittest

from automon.helpers.subprocessWrapper import Run


class TestRun(unittest.TestCase):
    def test_text(self):
        run = Run()
        run.run('ls', text=True)
        self.assertEqual(type(run.stdout), bytes)


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.mac import os_is_mac
from automon.integrations.mac.wdutil import WdutilClient


class MyTestCase(unittest.TestCase):
    if WdutilClient().is_ready():
        def test_something(self):
            client = WdutilClient()
            self.assertTrue(client.run('info'))
            self.assertTrue(client.info())
            pass


if __name__ == '__main__':
    unittest.main()



import os

from automon import log
from automon import environ

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class SwiftConfig:
    OPENSTACK_USERNAME = environ('OPENSTACK_USERNAME', '')
    OPENSTACK_PASSWORD = environ('OPENSTACK_PASSWORD', '')
    OPENSTACK_AUTH_URL = environ('OPENSTACK_AUTH_URL', '')
    OPENSTACK_PROJECT_ID = environ('OPENSTACK_PROJECT_ID', '')
    OPENSTACK_PROJECT_NAME = environ('OPENSTACK_PROJECT_NAME', '')
    OPENSTACK_USER_DOMAIN_NAME = environ('OPENSTACK_USER_DOMAIN_NAME', '')
    OPENSTACK_PROJECT_DOMAIN_ID = environ('OPENSTACK_PROJECT_DOMAIN_ID', '')
    OPENSTACK_REGION_NAME = environ('OPENSTACK_REGION_NAME', 'RegionOne')
    OPENSTACK_INTERFACE = environ('OPENSTACK_INTERFACE', 'public')
    OPENSTACK_IDENTITY_API_VERSION = environ('OPENSTACK_IDENTITY_API_VERSION', 3)
    SWIFTCLIENT_INSECURE = environ('SWIFTCLIENT_INSECURE', True)

    def __init__(self):
        pass

    def is_ready(self):
        if not self.OPENSTACK_USERNAME:
            logger.error(f'missing OPENSTACK_USERNAME')
            return False
        if not self.OPENSTACK_PASSWORD:
            logger.error(f'missing OPENSTACK_PASSWORD')
            return False
        if not self.OPENSTACK_AUTH_URL:
            logger.error(f'missing OPENSTACK_AUTH_URL')
            return False
        if not self.OPENSTACK_PROJECT_ID:
            logger.error(f'missing OPENSTACK_PROJECT_ID')
            return False
        if not self.OPENSTACK_PROJECT_NAME:
            logger.error(f'missing OPENSTACK_PROJECT_NAME')
            return False
        if not self.OPENSTACK_USER_DOMAIN_NAME:
            logger.error(f'missing OPENSTACK_USER_DOMAIN_NAME')
            return False
        if not self.OPENSTACK_PROJECT_DOMAIN_ID:
            logger.error(f'missing OPENSTACK_PROJECT_DOMAIN_ID')
            return False
        if not self.OPENSTACK_REGION_NAME:
            logger.error(f'missing OPENSTACK_REGION_NAME')
            return False
        if not self.OPENSTACK_INTERFACE:
            logger.error(f'missing OPENSTACK_INTERFACE')
            return False
        if not self.OPENSTACK_IDENTITY_API_VERSION:
            logger.error(f'missing OPENSTACK_IDENTITY_API_VERSION')
            return False
        if not self.SWIFTCLIENT_INSECURE:
            logger.error(f'missing SWIFTCLIENT_INSECURE')
            return False

        return True

    def __eq__(self, other):
        if not isinstance(other, SwiftConfig):
            logger.warning(f'Not implemented')
            return NotImplemented

        return self.OPENSTACK_USERNAME == other.OPENSTACK_USERNAME and \
            self.OPENSTACK_PASSWORD == other.OPENSTACK_PASSWORD and \
            self.OPENSTACK_AUTH_URL == other.OPENSTACK_AUTH_URL and \
            self.OPENSTACK_PROJECT_ID == other.OPENSTACK_PROJECT_ID and \
            self.OPENSTACK_PROJECT_NAME == other.OPENSTACK_PROJECT_NAME and \
            self.OPENSTACK_PROJECT_DOMAIN_ID == other.OPENSTACK_PROJECT_DOMAIN_ID



import re
import random
import datetime
import statistics

import automon
from automon import log
from automon.helpers.sleeper import Sleeper
from automon.integrations.seleniumWrapper import SeleniumBrowser, ChromeWrapper

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class FacebookGroups(object):
    _xpath_close_login_popup = '/html/body/div[1]/div/div[1]/div/div[5]/div/div/div[1]/div/div[2]/div/div/div/div[1]/div'
    _xpath_content_unavailable = '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div/div/div[1]/div[2]/div[1]/h2/span'
    _xpath_title = '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[1]/div[2]/div/div/div/div/div[1]/div/div/div/div/div/div[1]/h1/span/a'
    _xpath_temporarily_blocked = '/html/body/div[1]/div[2]/div[1]/div/div/div[1]/div/div[2]'
    _xpath_must_login = '/html/body/div[1]/div[1]/div[1]/div/div[2]/div/div'
    _xpath_blocked_by_login = ''
    _xpath_browser_not_supported = ''

    _xpath_creation_date = [
        '/html/body/div[1]/div/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[3]/div/div/div[2]/div/div/span',
    ]
    _xpath_history = '/html/body/div[1]/div/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div[3]/div/div/div[2]/div/div[2]/span/span'
    _xpath_members = [
        '/html/body/div[1]/div/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[2]/div/div/div[2]/div/div[1]',
        '/html/body/div[1]/div/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[2]/div/div/div[2]/div/div[1]/span',
        '//*[@id="mount_0_0_S5"]/div/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[2]/div/div/div[2]/div/div[1]/span',
    ]
    _xpath_posts_monthly = '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[1]/div/div/div[2]/div/div[2]'
    _xpath_posts_today = '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[3]/div/div/div/div/div/div[2]/div/div[1]/div/div/div[2]/div/div[1]'
    _xpath_privacy = '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div[1]/div/div/div[2]/div/div[1]'
    _xpath_privacy_details = '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div[1]/div/div/div[2]/div/div[2]'
    _xpath_visible = '/html/body/div[1]/div/div[1]/div/div[3]/div/div/div[1]/div[1]/div[4]/div/div/div/div/div/div[1]/div/div/div/div/div/div[2]/div[2]/div[2]/div/div/div[2]/div/div[1]'

    RATE_PER_MINUTE = 2
    RATE_COUNTER = []
    LAST_REQUEST = None
    WAIT_BETWEEN_RETRIES = random.choice(range(1, 60))

    def __init__(self, url: str = None):
        """Facebook Groups object

        Depends on Selenium"""
        self._url = url

        self._browser = SeleniumBrowser()

        self._content_unavailable = None
        self._creation_date = None
        self._creation_date_timestamp = None
        self._history = None
        self._members = None
        self._members_count = None
        self._posts_monthly = None
        self._posts_monthly_count = None
        self._posts_today = None
        self._posts_today_count = None
        self._privacy = None
        self._privacy_details = None
        self._title = None
        self._visible = None
        self._browser_not_supported = None

        self._check_blocked_by_login = None
        self._check_temporarily_blocked = None
        self._check_something_went_wrong = None

    def average_rate(self):
        if self.RATE_COUNTER:
            avg_seconds = round(statistics.mean(self.RATE_COUNTER), 1)
            avg_minutes = round(avg_seconds / 60, 1)
            avg_hours = round(avg_minutes / 60, 1)
            logger.info(
                f'total requests={len(self.RATE_COUNTER)} :: '
                f'{avg_seconds=} :: '
                f'{avg_minutes=} :: '
                f'last request {self.RATE_COUNTER[-1]} sec')
            return avg_seconds
        return 0

    def check_blocked_by_login(self):

        if self._check_blocked_by_login is not None:
            return self._check_blocked_by_login

        element = self._browser.wait_for_xpath(value=self._xpath_blocked_by_login, timeout=3)
        if element:
            element = element.text

        method = 'by XPATH'

        if not element:
            element = self._browser.find_all_with_beautifulsoup(
                string='You must log in to continue.',
                case_sensitive=True
            )
            if element:
                element = element[0].text

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._blocked_by_login = element
        return element

    def check_browser_not_supported(self):

        if self._browser_not_supported is not None:
            return self._browser_not_supported

        element = self._browser.wait_for_xpath(
            value=self._xpath_browser_not_supported,
            timeout=3)
        if element:
            element = element.text

        method = 'by XPATH'

        if not element:
            element = self._browser.find_all_with_beautifulsoup(
                string='This browser is not supported',
                case_sensitive=True
            )
            if element:
                element = element[0].text

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._browser_not_supported = element
        return element

    def check_close_login_popup(self):
        element = self._browser.wait_for_xpath(value=self._xpath_close_login_popup, timeout=3)
        method = 'by XPATH'

        if not element:
            element = self._browser.find_anything(
                match='Close',
                value='[aria-label="Close"]',
                by=self._browser.by.CSS_SELECTOR,
            )
            if element:
                element = element[0]

            method = 'by SEARCH'

        if element:
            element.click()

        logger.debug(f':: {method} :: {element}')
        return element

    def check_content_unavailable(self):
        """This content isn't available right now"""

        if self._content_unavailable is not None:
            return self._content_unavailable

        element = self._browser.wait_for_xpath(value=self._xpath_content_unavailable, timeout=3)
        if element:
            element = element.text
        method = 'by XPATH'

        if not element:
            element = self._browser.find_all_with_beautifulsoup(
                string="This content isn't available right now",
                case_sensitive=True
            )
            if element:
                element = element[0]

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._content_unavailable = element
        return element

    def check_something_went_wrong(self):
        """Happens when facebook server has issues"""

        if self._check_something_went_wrong:
            return self._check_something_went_wrong

        element = self._browser.wait_for_anything_with_beautifulsoup(
            match='Sorry, something went wrong.',
            string='Sorry, something went wrong.',
            case_sensitive=True,
            timeout=3,
        )
        if element:
            element = element[0]

        method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        return element

    def check_temporarily_blocked(self):
        """check if your IP is blocked by facebook"""

        if self._check_temporarily_blocked:
            return self._check_temporarily_blocked

        element = self._browser.wait_for_xpath(value=self._xpath_temporarily_blocked, timeout=3)
        method = 'by XPATH'
        if element:
            element = element.text

        logger.debug(f':: {method} :: {element}')
        return element

    def creation_date(self):

        if self._creation_date is not None:
            return self._creation_date

        element = [
            x for x in
            [self._browser.wait_for_xpath(value=xpath, timeout=3) for xpath in self._xpath_creation_date] if x]
        if element:
            element = element[0].text
            element = element.splitlines()[0]
        method = 'by XPATH'

        if not element:
            re_matches = [
                'Created \d+ year[s]? ago',
                'Created a year ago',
            ]

            for re_match in re_matches:

                element = self._browser.find_elements_with_beautifulsoup(
                    match=re_match,
                    name='span',
                    case_sensitive=True,
                )

                if element:
                    element = element[0].text
                    element = re.compile(re_match).match(element)[0]
                    break

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._creation_date = element

        if not element:
            raise f'{element=}'

        return element

    def creation_date_timestamp(self):

        if self._creation_date_timestamp is not None:
            return self._creation_date_timestamp

        if self._creation_date is not None or self.creation_date():
            # TODO: convert date to datetime timestamp
            logger.warn(NotImplemented)
            return

    @property
    def current_url(self):
        return self._browser.current_url

    def current_rate_too_fast(self):
        if self.average_rate() == 0:
            logger.info(f'current_rate_too_fast :: False')
            return False

        if self.average_rate() < self.rate_per_minute:
            logger.info(f'current_rate_too_fast :: True')
            return True

        return False

    @staticmethod
    def error_parsing(error, enable_stacktrace: bool = False) -> tuple:
        """parses selenium exeption error"""

        error_parsed = f'{error}'.splitlines()
        error_parsed = [f'{x}'.strip() for x in error_parsed]
        message = error_parsed[0]
        session = None
        stacktrace = None
        if len(error_parsed) > 1:
            session = error_parsed[1]
            stacktrace = error_parsed[2:]
            stacktrace = ' '.join(stacktrace)

        if enable_stacktrace:
            return message, session, stacktrace

        return message, session, 'disabled'

    def get(self, url: str) -> bool:
        """get url"""

        now = datetime.datetime.now().timestamp()

        if self.LAST_REQUEST:
            seconds_elapsed_since_last_request = abs(self.LAST_REQUEST - now)
            self.RATE_COUNTER.append(round(seconds_elapsed_since_last_request, 1))
            self.LAST_REQUEST = round(now, 1)
        else:
            self.LAST_REQUEST = round(now, 1)

        result = self._browser.get(url=url)
        logger.info(f'{result=} :: {url} :: {round(len(self._browser.webdriver.page_source) / 1024)} KB')
        return result

    def get_about(self, rate_limiting: bool = True):
        """get about page"""

        about = self._browser._urllib.parse.urljoin(self.url + '/', 'about')

        if rate_limiting:
            result = self.get_with_rate_limiter(url=about)
        else:
            result = self.get(url=about)

        logger.info(f'{about} :: {result=}')

        return result

    def get_with_rate_limiter(
            self,
            url: str,
            retry: int = 0,
            retries: int = 5,
            wait_between_retries: int = None,
            rate_per_minute: int = None,
    ) -> bool:
        """get with rate dynamic limit"""

        if wait_between_retries:
            self.WAIT_BETWEEN_RETRIES = wait_between_retries

        if rate_per_minute:
            self.RATE_PER_MINUTE = rate_per_minute

        result = None
        while retry < retries:

            if self.rate_limited():
                self.rate_limit_increase()
                Sleeper.seconds(seconds=self.WAIT_BETWEEN_RETRIES)
                logger.debug(f'get_with_rate_limiter :: retrying :: {url} :: {retry=} :: {retries=}')
            else:
                self.rate_limit_decrease()

            result = self.get(url=url)
            logger.info(f'get_with_rate_limiter :: {result}')
            return result

            retry = retry + 1

        logger.error(f'get_with_rate_limiter :: error :: {url}')
        self.screenshot_error()
        return result

    def quit(self):
        """quit selenium"""

        if self._browser:
            logger.info(f'quit :: {self._browser}')
            return self._browser.quit()

    @property
    def rate_per_minute(self) -> int:
        rate = int(60 / self.RATE_PER_MINUTE)
        logger.info(f'{rate=} sec')
        return rate

    def history(self):

        if self._history is not None:
            return self._history

        element = self._browser.wait_for_xpath(value=self._xpath_history, timeout=3)
        if element:
            element = element.text
            if 'See more' in element:
                element = element.split('See more')
                element = element[0]
        method = 'by XPATH'

        if not element:
            re_matches = [
                'Group created on \w+ \d+, \d+',
            ]

            for re_match in re_matches:

                elements = self._browser.find_elements_with_beautifulsoup(
                    match=re_match,
                    name='span',
                    case_sensitive=True,
                )

                if elements:

                    for element in elements:
                        element = element.text
                        element = re.compile(re_match).match(element)[0]

                        if element:
                            break

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._history = element

        if not element:
            raise f'{element=}'

        return element

    def rate_limit_decrease(self, multiplier: int = 0.75):
        before = self.WAIT_BETWEEN_RETRIES
        after = abs(int(self.WAIT_BETWEEN_RETRIES * multiplier))

        self.WAIT_BETWEEN_RETRIES = after

        if self.WAIT_BETWEEN_RETRIES == 0:
            self.WAIT_BETWEEN_RETRIES = 1

        logger.info(f'{before=} :: {after=} :: {multiplier=}')
        return after

    def rate_limit_increase(self, multiplier: int = 2):
        if self.WAIT_BETWEEN_RETRIES == 0:
            self.WAIT_BETWEEN_RETRIES = random.choice(range(1, 60))

        before = self.WAIT_BETWEEN_RETRIES
        after = abs(int(self.WAIT_BETWEEN_RETRIES * multiplier))

        self.WAIT_BETWEEN_RETRIES = after

        logger.info(f'{before=} :: {after=} :: {multiplier=}')
        return after

    def rate_limited(self):
        """rate limit checker"""

        if self.current_rate_too_fast():
            logger.info(f'rate_limited :: True')
            return True

        if self.check_temporarily_blocked() or self.must_login():
            logger.info(f'rate_limited :: True')
            return True

        logger.info(f'rate_limited :: False')

        return False

    def reset_cache(self):
        self._content_unavailable = None
        self._creation_date = None
        self._creation_date_timestamp = None
        self._history = None
        self._members = None
        self._members_count = None
        self._posts_monthly = None
        self._posts_monthly_count = None
        self._posts_today = None
        self._posts_today_count = None
        self._privacy = None
        self._privacy_details = None
        self._title = None
        self._visible = None
        self._blocked_by_login = None
        self._browser_not_supported = None

    def reset_rate_counter(self):
        self.RATE_COUNTER = []
        logger.info(f'reset_rate_counter :: {self.RATE_COUNTER}')
        return self.RATE_COUNTER

    def restart(self):
        """quit and start new instance of selenium"""

        if self._browser:
            self.quit()
        logger.info(f'restart :: {self._browser}')
        return self.start()

    def run(self):
        """run selenium browser"""

        if self._browser:
            logger.info(f'run :: {self._browser}')
            return self._browser.run()

    def screenshot(self, filename: str = 'screenshot.png'):
        if self._browser.save_screenshot(filename=filename, folder='.'):
            logger.debug(f'screenshot :: done')
            return True
        return False

    def screenshot_error(self):
        """get error screenshot"""

        if self.screenshot(filename='screenshot-error.png'):
            logger.debug(f'screenshot_error :: done')
            return True
        return False

    def screenshot_success(self):
        """get success screenshot"""

        if self.screenshot(filename='screenshot-success.png'):
            logger.debug(f'screenshot_success :: done')
            return True
        return False

    def set_url(self, url: str) -> str:
        """set new url"""

        self._url = url
        logger.debug(f'set_url :: {self.url=}')
        return self.url

    def start(self, headless: bool = True, random_user_agent: bool = False, set_user_agent: str = None):
        """start new instance of selenium"""

        self._browser.config.webdriver_wrapper = ChromeWrapper()

        if headless:
            self._browser.config.webdriver_wrapper.enable_headless()
            self._browser.config.webdriver_wrapper.set_locale_experimental()
        else:
            self._browser.config.webdriver_wrapper.set_locale_experimental()

        if random_user_agent:
            self._browser.config.webdriver_wrapper.set_user_agent(
                self._browser.get_random_user_agent()
            )
        elif set_user_agent:
            self._browser.config.webdriver_wrapper.set_user_agent(
                set_user_agent
            )

        logger.info(f'start :: {self._browser}')
        browser = self._browser.run()
        self._browser.config.webdriver_wrapper.set_window_size(width=1920 * 0.6, height=1080)
        return browser

    def stop(self):
        """alias to quit"""

        return self.quit()

    def members(self):

        if self._members is not None:
            return self._members

        for xpath in self._xpath_members:
            element = self._browser.wait_for_xpath(value=xpath, timeout=3)

            if element:
                element = element.text
                break

        method = 'by XPATH'

        if not element:
            element = self._browser.find_all_with_beautifulsoup(
                string='[0-9]?+[,]?[0-9]+ total member[s]?',
                case_sensitive=True
            )
            if element:
                element = element[0].text

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._members = element

        if not element:
            raise f'{element=}'

        return element

    def members_count(self):

        if self._members_count is not None:
            return self._members_count

        if self._members is not None or self.members():
            count = [x for x in self._members]
            count = [x for x in count if x in [str(x) for x in range(0, 10)]]
            if count:
                self._members_count = int(''.join(count)) if count else 0

                logger.debug(self._members_count)
                return self._members_count

    def must_login(self):
        element = self._browser.wait_for_xpath(self._xpath_must_login, timeout=3)
        method = 'by XPATH'
        if element:
            element = element.text
        logger.debug(f':: {method} :: {element}')
        return element

    def posts_monthly(self):

        if self._posts_monthly is not None:
            return self._posts_monthly

        element = self._browser.wait_for_xpath(value=self._xpath_posts_monthly, timeout=3)
        if element:
            element = element.text
        method = 'by XPATH'

        if not element:
            element = self._browser.find_all_with_beautifulsoup(
                string='in the last month',
                case_sensitive=True
            )

            if element:
                element = element[0].text

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._posts_monthly = element

        if not element:
            raise f'{element=}'

        return element

    def posts_monthly_count(self):

        if self._posts_monthly_count is not None:
            return self._posts_monthly_count

        if self._posts_monthly is not None or self.posts_monthly():
            count = [x for x in self._posts_monthly]
            count = [x for x in count if x in [str(x) for x in range(0, 10)]]
            if count:
                self._posts_monthly_count = int(''.join(count)) if count else 0

                logger.debug(self._posts_monthly_count)
                return self._posts_monthly_count

    def posts_today(self):

        if self._posts_today is not None:
            return self._posts_today

        element = self._browser.wait_for_xpath(value=self._xpath_posts_today, timeout=3)
        if element:
            element = element.text
        method = 'by XPATH'

        if not element:
            element = self._browser.find_all_with_beautifulsoup(
                string='new post[s]? today',
                case_sensitive=True
            )
            if element:
                element = element[0].text

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._posts_today = element

        if not element:
            raise f'{element=}'

        return element

    def posts_today_count(self):

        if self._posts_today_count is not None:
            return self._posts_today_count

        if self._posts_today is not None or self.posts_today():
            count = [x for x in self._posts_today]
            count = [x for x in count if x in [str(x) for x in range(0, 10)]]
            if count:
                self._posts_today_count = int(''.join(count)) if count else 0

                logger.debug(self._posts_today_count)
                return self._posts_today_count

    def privacy(self):

        if self._privacy is not None:
            return self._privacy

        element = self._browser.wait_for_xpath(value=self._xpath_privacy, timeout=3)
        if element:
            element = element.text
        method = 'by XPATH'

        if not element:
            re_matches = [
                'Public',
                'Private',
                'Visible',
            ]

            for re_match in re_matches:
                element = self._browser.find_all_with_beautifulsoup(
                    name='span',
                    string=re_match,
                    case_sensitive=True
                )
                if element:
                    element = element[0].text
                    break

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._privacy = element

        if not element:
            raise f'{element=}'

        return element

    def privacy_details(self):

        if self._privacy_details is not None:
            return self._privacy_details

        element = self._browser.wait_for_xpath(value=self._xpath_privacy_details, timeout=3)
        if element:
            element = element.text
        method = 'by XPATH'

        if not element:
            re_matches = [
                "Anyone can see who's in the group and what they post.",
                "Only members can see who's in the group and what they post.",
            ]

            for re_match in re_matches:
                element = self._browser.find_all_with_beautifulsoup(
                    string=re_match,
                    case_sensitive=True
                )
                if element:
                    element = element[0].text
                    break

            method = 'by SEARCH'

        self._privacy_details = element
        logger.debug(f':: {method} :: {element}')

        if not element:
            raise f'{element=}'

        return element

    def title(self) -> str:

        if self._title is not None:
            return self._title

        self._title = self._browser.webdriver.title

        return_empty = [
            'Log into Facebook',
            'www.facebook.com'
        ]

        if self._title in return_empty:
            self._title = ''

        if self._title:
            self._title = self._title.split('|')
            self._title = self._title[0]
            self._title = self._title.strip()
        logger.debug(self._title)
        return self._title

    def to_dict(self):
        return dict(
            creation_date=self.creation_date(),
            creation_date_timestamp=self.creation_date_timestamp(),
            history=self.history(),
            members=self.members(),
            members_count=self.members_count(),
            posts_monthly=self.posts_monthly(),
            posts_monthly_count=self.posts_monthly_count(),
            posts_today=self.posts_today(),
            posts_today_count=self.posts_today_count(),
            privacy=self.privacy(),
            privacy_details=self.privacy_details(),
            title=self.title(),
            url=self.url,
            visible=self.visible(),
            check_blocked_by_login=self.check_blocked_by_login(),
            check_browser_not_supported=self.check_browser_not_supported(),
            check_content_unavailable=self.check_content_unavailable(),
        )

    def to_empty(self):
        return dict(
            creation_date=None,
            creation_date_timestamp=None,
            history=None,
            members=None,
            members_count=None,
            posts_monthly=None,
            posts_monthly_count=None,
            posts_today=None,
            posts_today_count=None,
            privacy=None,
            privacy_details=None,
            title=None,
            url=self.url,
            visible=None,
            check_blocked_by_login=self.check_blocked_by_login(),
            check_browser_not_supported=self.check_browser_not_supported(),
            check_content_unavailable=self.check_content_unavailable(),
        )

    @property
    def url(self) -> str:
        return self.url_cleaner(self._url)

    @staticmethod
    def url_cleaner(url: str):
        """simple url cleaner"""

        if not url:
            return
        if url[-1] == '/':
            url = url[:-1]
        # https://m.facebook.com/groups/ -> https://www.facebook.com/groups/
        if '://m.' in url:
            url = url.replace('://m.', '://www.')
        return url

    def visible(self) -> str:

        if self._visible is not None:
            return self._visible

        element = self._browser.wait_for_xpath(value=self._xpath_visible, timeout=3)
        if element:
            element = element.text
        method = 'by XPATH'

        if not element:
            re_matches = [
                'Anyone can find this group.',
            ]

            for re_match in re_matches:
                element = self._browser.find_all_with_beautifulsoup(
                    string=re_match,
                    case_sensitive=True
                )
                if element:
                    element = element[0].text
                    break

            method = 'by SEARCH'

        logger.debug(f':: {method} :: {element}')
        self._visible = element

        if not element:
            raise f'{element=}'

        return element



class V1(object):

    def __init__(self, host):
        self.endpoint = host

    @property
    def api(self):
        self.endpoint += '/api/v1'
        return self

    @property
    def validate(self):
        self.endpoint += '/validate'
        return self



import os
import csv

from automon import log
from automon.helpers import Run

logger = log.logging.getLogger(__name__)
logger.setLevel(level=log.DEBUG)


class KeepassEntry(object):
    ENTRY_FORMAT: dict = {
        'Group': 0,
        'Title': 1,
        'Username': 2,
        'Password': 3,
        'URL': 4,
        'Notes': 5,
        'TOPT': 6,
        'Icon': 7,
        'Last Modified': 8,
        'Created': 9,
    }

    def __init__(self, cvs_row: list):
        self.Group: str = cvs_row[self.ENTRY_FORMAT['Group']]
        self.Title: str = cvs_row[self.ENTRY_FORMAT['Title']]
        self.Username: str = cvs_row[self.ENTRY_FORMAT['Username']]
        self.Password: str = cvs_row[self.ENTRY_FORMAT['Password']]
        self.URL: str = cvs_row[self.ENTRY_FORMAT['URL']]
        self.Notes: str = cvs_row[self.ENTRY_FORMAT['Notes']]
        self.TOPT: str = cvs_row[self.ENTRY_FORMAT['TOPT']]
        self.Icon: str = cvs_row[self.ENTRY_FORMAT['Icon']]
        self.LastModified: str = cvs_row[self.ENTRY_FORMAT['Last Modified']]
        self.Created: str = cvs_row[self.ENTRY_FORMAT['Created']]

        self.pass_name = '/'.join(f'{self.Group}/{self.Title}'.split('/')[1:])

    def __str__(self):
        return f'{self.pass_name} :: {self.Username} :: {self.URL}'

    def to_pass(self):
        return f"""{self.Password}
Title: {self.Title}
Username: {self.Username}
URL: {self.URL}
TOPT: {self.TOPT}
Icon: {self.Icon}
Created: {self.Created}
Last Modified: {self.LastModified}
Notes: {self.Notes}
"""


class KeepassClient(object):

    def __init__(self):
        self.database_csv_path = None

    def read_database_csv(self, database_csv_path: str, delimiter: str = ',', quotechar: str = '"') -> [KeepassEntry]:
        if self.set_database_csv_path(database_csv_path=database_csv_path):
            logger.debug(f'KeepassClient :: read_database_csv :: read')
            with open(self.database_csv_path, 'r') as csvfile:
                return [KeepassEntry(pw) for pw in csv.reader(csvfile, delimiter=delimiter, quotechar=quotechar)][1:]

        raise OSError(f'KeepassClient :: read_database_csv :: ERROR :: {database_csv_path}')

    def set_database_csv_path(self, database_csv_path: str) -> True:
        if os.path.exists(database_csv_path):
            self.database_csv_path = database_csv_path
            logger.debug(
                f'KeepassClient :: set_database_csv_path :: {database_csv_path} :: {os.stat(database_csv_path)}')
            return True

        raise FileNotFoundError(f'KeepassClient :: set_database_csv_path :: ERROR :: {database_csv_path}')


class PassClient(object):

    def __init__(self):
        self.runner = Run()

    def save(self, pass_name: str, password: str):
        with open('password_temp.txt', 'w') as password_file:
            password_file.write(password)

        command = (f'pass insert -f -m "{pass_name}" < password_temp.txt')
        self.runner.run(command=command, shell=True)



from automon.integrations.requestsWrapper import RequestsClient
from automon import log

from .config import DatadogConfigRest
from .api import V1, V2

logger = log.logging.getLogger(__name__)
logger.setLevel(log.logging.DEBUG)


class DatadogClientRest(object):

    def __init__(self, host: str = None, api_key: str = None):
        self.config = DatadogConfigRest(host=host, api_key=api_key)
        self.requests = RequestsClient()

    async def is_ready(self):
        if await self.config.is_ready():
            if await self.validate():
                return True
        logger.error(f'client not ready')

    async def log(self, ddsource: str, hostname: str, service: str, message: str, ddtags: str = 'env:test,version:0.1'):
        url = V2(self.config.host_log).api.logs.endpoint

        log = {
            "ddsource": ddsource,
            "ddtags": ddtags,
            "hostname": hostname,
            "service": service,
            'message': message
        }

        logger.debug(log)

        response = await self.requests.post(url=url, json=log)
        response_log = await self.requests.to_dict()

        return response_log

    async def validate(self):
        url = V1(self.config.host).api.validate.endpoint

        self.requests.session.headers.update(await self.config.headers())
        response = await self.requests.get(url=url)
        response_validate = await self.requests.to_dict()

        return response_validate



import profile
import asyncio
import datetime
import tracemalloc

import pandas as pd
import numpy as np

from automon import log
from automon.helpers.sleeper import Sleeper
from automon.integrations.facebook import FacebookGroups
from automon.integrations.google.sheets import GoogleSheetsClient

log.logging.getLogger('google_auth_httplib2').setLevel(log.logging.ERROR)
log.logging.getLogger('googleapiclient.discovery').setLevel(log.logging.ERROR)
log.logging.getLogger('googleapiclient.discovery_cache').setLevel(log.logging.ERROR)
log.logging.getLogger('urllib3.connectionpool').setLevel(log.logging.ERROR)
log.logging.getLogger('selenium.webdriver.common.service').setLevel(log.logging.ERROR)
log.logging.getLogger('selenium.webdriver.remote.remote_connection').setLevel(log.logging.ERROR)
log.logging.getLogger('selenium.webdriver.common.selenium_manager').setLevel(log.logging.ERROR)

log.logging.getLogger('automon.integrations.seleniumWrapper.browser').setLevel(log.logging.CRITICAL)
log.logging.getLogger('automon.integrations.seleniumWrapper.webdriver_chrome').setLevel(log.logging.INFO)
log.logging.getLogger('automon.integrations.google.sheets.client').setLevel(log.logging.INFO)
log.logging.getLogger('automon.integrations.facebook.groups').setLevel(log.logging.DEBUG)
log.logging.getLogger('automon.integrations.requestsWrapper.client').setLevel(log.logging.INFO)
log.logging.getLogger('automon.helpers.sleeper').setLevel(log.logging.INFO)

tracemalloc.start()

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)

SHEET_NAME = 'Automated Count DO NOT EDIT'
SHEET_NAME_INGEST = 'URLS TO INGEST'

sheets_client = GoogleSheetsClient(
    worksheet=SHEET_NAME,
)

facebook_group_client = FacebookGroups()


async def get_facebook_info(url: str):
    dict = {}

    if not url:
        return dict

    await facebook_group_client.start(
        headless=True,
        random_user_agent=True,
    )

    await facebook_group_client.set_url(url=url)

    try:
        result = await facebook_group_client.get_about(rate_limiting=False)
        dict = await facebook_group_client.to_dict()
        await facebook_group_client.quit()
    except Exception as e:
        logger.error(f'{e}')

    return dict


async def url_cleaner(url: str):
    if not url:
        return
    if url[-1] == '/':
        url = url[:-1]
    return url


async def merge_urls():
    await sheets_client.get(
        ranges='AUG23 AUDIT!A:Z',
        fields="sheets/data/rowData/values/hyperlink",
    )

    data = sheets_client.response['sheets'][0]['data'][0]['rowData']
    # expand nested data
    links = []
    for x in data:
        if x:
            links.append(
                x['values'][0]['hyperlink']
            )

    df_Shelley = pd.DataFrame(data=links, columns=['url'])

    await sheets_client.get()
    await sheets_client.get_values(
        range=f'{SHEET_NAME}!A:Z'
    )

    sheet_values = sheets_client.values
    sheet_columns = sheet_values[0]
    sheet_data = sheet_values[1:]

    df = pd.DataFrame(data=sheet_data, columns=sheet_columns)
    df = df.dropna(subset=['url'])

    # merge both lists or urls
    df = pd.merge(df, df_Shelley, how='outer', on='url')
    df = df.drop_duplicates(subset=['url'], keep='first')
    return df


async def batch_processing(sheet_index: int, df: pd.DataFrame):
    # df_results = df['url'].dropna().apply(
    #     lambda url: get_facebook_info(url=url)
    # )

    df_index = df['url'].dropna().index
    df_url = df['url'].dropna().iloc[0]
    df_results = await get_facebook_info(url=df_url)
    df_results = pd.DataFrame([df_results])

    df = df.reset_index()

    todays_date = datetime.datetime.now().date()
    monthly = f'{todays_date.year}-{todays_date.month}'

    assert df['url'].iloc[0] == df_results['url'].iloc[0]

    # create columns
    df[f'url'] = df_results['url']
    df[f'{monthly}'] = df_results['members_count']
    df[f'last_updated'] = monthly
    df[f'title'] = df_results['title']
    df[f'content_unavailable'] = df_results['content_unavailable']
    df[f'creation_date'] = df_results['creation_date']
    df[f'creation_date_timestamp'] = df_results['creation_date_timestamp']
    df[f'history'] = df_results['history']
    df[f'members_count'] = df_results['members_count']
    df[f'posts_monthly_count'] = df_results['posts_monthly_count']
    df[f'posts_today_count'] = df_results['posts_today_count']
    df[f'privacy'] = df_results['privacy']
    df[f'visible'] = df_results['visible']

    # set dtype to Int32
    df[f'{monthly}'] = df[f'{monthly}'].astype('Int32')
    df[f'creation_date_timestamp'] = df[f'creation_date_timestamp'].astype('Int32')
    df[f'members_count'] = df[f'members_count'].astype('Int32')
    df[f'posts_monthly_count'] = df[f'posts_monthly_count'].astype('Int32')
    df[f'posts_today_count'] = df[f'posts_today_count'].astype('Int32')

    # order columns
    columns = [
        'url',
        'title',
        'creation_date',
        'creation_date_timestamp',
        'history',
        'privacy',
        'visible',
        'content_unavailable',
        'last_updated',
        'posts_monthly_count',
        'posts_today_count',
        'members_count',
    ]

    sheet_index_df = df['index'].loc[0]
    assert sheet_index == sheet_index_df
    df = df.drop(columns='index')

    # add all other columns
    df_columns = df.columns.tolist()
    columns.extend(
        [x for x in df_columns if x not in columns]
    )

    # finally add today's date
    if f'{monthly}' not in columns:
        columns.append(
            f'{monthly}',
        )

    df = df.loc[:, columns]
    df = df.fillna(np.nan).replace([np.nan], [None])

    update_columns = await sheets_client.update(
        range=f'{SHEET_NAME}!A1:Z',
        values=[columns],
    )

    update = await sheets_client.update(
        range=f'{SHEET_NAME}!A{sheet_index_df}:Z',
        values=[x for x in df.values.tolist()]
    )

    logger.info(f'{sheet_index_df}: {[x for x in df.values.tolist()]}')

    return df


async def memory_profiler():
    snapshot = tracemalloc.take_snapshot()
    top_stats = snapshot.statistics("lineno")

    df_memory_profile = pd.DataFrame([
        dict(size_B=stat.size, count=stat.count, file=stat.traceback._frames[0][0],
             file_line=stat.traceback._frames[0][1]) for stat in top_stats
    ])
    df_memory_profile.sort_values(by='size_B', ascending=False)
    df_memory_profile['size_KB'] = df_memory_profile['size_B'].apply(
        lambda B: round(B / 1024)
    )
    df_memory_profile['size_MB'] = df_memory_profile['size_KB'].apply(
        lambda KB: round(KB / 1024)
    )
    cols = df_memory_profile.columns.tolist()
    cols.sort()
    df_memory_profile = df_memory_profile.loc[:, cols]

    logger.debug(
        f"total memory used: {df_memory_profile['size_MB'].sum()} MB; "
        f'most memory used: '
        f'{df_memory_profile.iloc[0].to_dict()}'
    )

    return df_memory_profile


async def expensive_state_keeping():
    """fetch sheet data"""
    if not await sheets_client.authenticate():
        return

    # merge urls from audit sheet
    # df_audit = await merge_urls()
    # df_audit = df_audit.fillna(np.nan).replace([np.nan], [None])
    # rows = []
    # rows.append(df_audit.columns.tolist())
    # rows.extend([x for x in df_audit.values.tolist()])
    # update = await sheets_client.update(
    #     range=f'{SHEET_NAME}!A:Z',
    #     values=rows
    # )

    # start processing
    await sheets_client.get_values(
        range=f'{SHEET_NAME}!A:Z'
    )

    sheet_values = sheets_client.values
    try:
        sheet_columns = sheet_values[0]
    except:
        return await expensive_state_keeping()
    sheet_data = sheet_values[1:]

    if sheet_columns and sheet_data:
        for row in sheet_data:
            if len(sheet_columns) > len(sheet_data[sheet_data.index(row)]):

                fix_length = row
                r = len(sheet_columns) - len(sheet_data[sheet_data.index(row)])
                for i in range(r):
                    fix_length.append(None)

                sheet_data[sheet_data.index(row)] = fix_length

    df = pd.DataFrame(data=sheet_data, columns=sheet_columns)
    df = df.dropna(subset=['url'])
    # set df index to match google sheet index numbering
    df.index = df.index + 2

    # drop duplicates
    df_duplicates_to_delete = df[
        (df.url.duplicated(keep='first'))
    ]

    for duplicate in df_duplicates_to_delete.iterrows():
        duplicate_index, duplicate_row = duplicate

        # clear row in sheet
        data_range = f'{SHEET_NAME}!{duplicate_index}:{duplicate_index}'
        result = await sheets_client.clear(range=data_range)
        # max 60/min
        Sleeper.seconds(f'WriteRequestsPerMinutePerUser', seconds=1)
        logger.info(result)
        df = df.drop(duplicate_index)

    # ingest urls from SHEET_NAME_INGEST
    await sheets_client.get_values(
        range=f'{SHEET_NAME_INGEST}!A:Z'
    )
    ingest_sheet_values = sheets_client.values
    if ingest_sheet_values:
        ingest_sheet_values = [x[0] if x else [] for x in ingest_sheet_values]
        ingest_sheet_values = [await url_cleaner(x) for x in ingest_sheet_values]
    df_ingest_sheet_values = pd.DataFrame(ingest_sheet_values)
    df_ingest_sheet_values.index = df_ingest_sheet_values.index + 1

    for ingest_data in df_ingest_sheet_values.iterrows():
        ingest_index, ingest_url = ingest_data
        ingest_url = ingest_url[0]

        if ingest_url not in df['url'].values:
            ingest_series = pd.Series({'url': ingest_url}).to_frame().T
            index_add_url = df.index[-1] + 1
            df.loc[index_add_url] = {'url': ingest_url}

            df.loc[index_add_url] = df.loc[index_add_url].fillna(np.nan).replace([np.nan], [None])

            values = [[x for x in df.loc[index_add_url].values.tolist()]]

            update = await sheets_client.update(
                range=f'{SHEET_NAME}!A{index_add_url}:Z',
                values=values
            )

            logger.info(
                f'{index_add_url}: {values}'
            )

        # clear url from ingest sheet
        data_range = f'{SHEET_NAME_INGEST}!{ingest_index}:{ingest_index}'
        clear = await sheets_client.clear(range=data_range)
        logger.info(f'{clear}')

    return df


async def main():
    df = await expensive_state_keeping()

    # start updating
    for data in df.iterrows():
        data_index, data_row = data

        df_batch = df.loc[data_index:data_index]

        # skip if last_updated is the current month
        todays_date = datetime.datetime.now().date()
        last_updated = f'{todays_date.year}-{todays_date.month}'
        if df_batch['last_updated'].iloc[0] == last_updated:
            # log.debug(f'skipping {data_index}, {data_row.to_dict()}')
            # df = expensive_state_keeping()
            continue

        try:
            logger.info(f'complete {round(data_index / len(df) * 100)}% {data_index}/{len(df)}')
            batch_result = await batch_processing(sheet_index=data_index, df=df_batch)
        except Exception as error:
            logger.error(f'{error}')
        df_memory = await memory_profiler()


if __name__ == '__main__':
    # loop = asyncio.get_event_loop()
    # loop.run_until_complete(main())
    asyncio.run(main())



class ActionClickException(Exception):
    pass


class ActionTypeException(Exception):
    pass


class ElementNotFoundException(Exception):
    pass


class XpathNotFoundException(ElementNotFoundException):
    pass



import os
import sys
import warnings
import selenium
import selenium.webdriver

from automon import log
from automon.helpers.osWrapper.environ import environ_list
from automon.integrations.seleniumWrapper.user_agents import SeleniumUserAgentBuilder

from .config_window_size import set_window_size

logger = log.logging.getLogger(__name__)
logger.setLevel(log.DEBUG)


class ChromeWrapper(object):

    def __init__(self):
        self._webdriver = None
        self._chrome_options = selenium.webdriver.ChromeOptions()
        self._chromedriver_path = environ_list('SELENIUM_CHROMEDRIVER_PATH')
        self._ChromeService = None
        self._window_size = set_window_size()
        self._enable_antibot_detection = None
        self._service_args = []

    def __repr__(self):
        if self._webdriver:
            return str(dict(
                name=self.webdriver.name,
                window_size=self.window_size,
                browserVersion=self.browserVersion,
                chromedriverVersion=self.chromedriverVersion,
                chromedriver_path=self.chromedriver_path,
                webdriver=self.webdriver,
            ))

        return f'{__class__}'

    @property
    def browserVersion(self):
        if self.webdriver:
            return self.webdriver.capabilities.get('browserVersion')

    @property
    def chrome_options(self):
        return self._chrome_options

    @property
    def chrome_options_arg(self):
        return self.chrome_options.arguments

    @property
    def chromedriver_path(self):
        for path in self._chromedriver_path:
            if os.path.exists(path):
                return path

        raise Exception('missing SELENIUM_CHROMEDRIVER_PATH')

    @property
    def chromedriverVersion(self):
        if self.webdriver:
            return self.webdriver.capabilities.get('chrome').get('chromedriverVersion')

    @property
    def ChromeService(self):
        return self._ChromeService

    @property
    def webdriver(self) -> selenium.webdriver.Chrome:
        return self._webdriver

    @property
    def window_size(self):
        return self._window_size

    def disable_automation_switch(self):
        """exclude the collection of enable-automation switches

        """
        warnings.warn(DeprecationWarning)

        self.chrome_options.add_experimental_option(
            "excludeSwitches", ["enable-automation"]
        )
        logger.debug(
            f'webdriver :: chrome :: '
            f'add_experimental_option :: '
            f'{dict(name="excludeSwitches", value=["enable-automation"])}'
        )

        return self

    def disable_AutomationControlled(self):
        """adding argument to disable the AutomationControlled flag

        """
        logger.debug(f'webdriver :: chrome :: add_argument :: --disable-blink-features=AutomationControlled')
        self.chrome_options.add_argument("--disable-blink-features=AutomationControlled")
        return self

    def disable_certificate_verification(self):
        logger.debug(f'webdriver :: chrome :: add_argument :: --ignore-certificate-errors')
        logger.warning('Certificates are not verified')
        self.chrome_options.add_argument('--ignore-certificate-errors')
        return self

    def disable_extensions(self):
        logger.debug(f'webdriver :: chrome :: add_argument :: --disable-extensions')
        self.chrome_options.add_argument("--disable-extensions")
        return self

    def disable_infobars(self):
        logger.debug(f'webdriver :: chrome :: add_argument :: --disable-infobars')
        self.chrome_options.add_argument("--disable-infobars")
        return self

    def disable_javascript_webdriver_prop(self):
        """changing the property of the navigator value for webdriver to undefined

        """
        self.webdriver.execute_script("Object.defineProperty(navigator, 'webdriver', {get: () => undefined})")
        return self

    def disable_notifications(self):
        """Pass the argument 1 to allow and 2 to block

        """
        self.chrome_options.add_experimental_option(
            "prefs", {"profile.default_content_setting_values.notifications": 2}
        )

        logger.debug(
            f'webdriver :: chrome :: '
            f'add_experimental_option :: '
            f'{dict(name="prefs", value={"profile.default_content_setting_values.notifications": 2})}'
        )

        return self

    def disable_sandbox(self):
        logger.debug(f'webdriver :: chrome :: add_argument :: --no-sandbox')
        self.chrome_options.add_argument('--no-sandbox')
        return self

    def disable_shm(self):
        logger.warning('Disabled shm will use disk I/O, and will be slow')
        logger.debug(f'webdriver :: chrome :: add_argument :: --disable-dev-shm-usage')
        self.chrome_options.add_argument('--disable-dev-shm-usage')
        return self

    def disable_userAutomationExtension(self):
        """turn-off userAutomationExtension

        """
        self.chrome_options.add_experimental_option(
            "useAutomationExtension", False
        )
        logger.debug(
            f'webdriver :: chrome :: '
            f'add_experimental_option :: '
            f'{dict(name="useAutomationExtension", value=False)}'
        )

        return self

    def download_chromedriver(self):
        versions = 'https://googlechromelabs.github.io/chrome-for-testing/latest-versions-per-milestone-with-downloads.json'
        raise

    def enable_antibot_detection(self):
        """Disabling the Automation Indicator WebDriver Flags

        """
        self.disable_AutomationControlled()
        self.disable_automation_switch()
        self.disable_userAutomationExtension()

        useragent = SeleniumUserAgentBuilder().get_top()
        self.set_user_agent(useragent)

        self._enable_antibot_detection = True
        return self

    def enable_bigshm(self):
        logger.warning('Big shm not yet implemented')
        return self

    def enable_defaults(self):
        self.enable_maximized()
        self.enable_logging()
        self.set_logging_level(level='ALL')
        return self

    def enable_fullscreen(self):
        logger.debug(f'webdriver :: chrome :: add_argument :: --start-fullscreen')
        self.chrome_options.add_argument("--start-fullscreen")
        return self

    def enable_headless(self):
        logger.debug(f'webdriver :: chrome :: add_argument :: headless')
        self.chrome_options.add_argument('headless')
        return self

    def enable_logging(self):
        logger.debug(f'webdriver :: chrome :: set_capability :: "goog:loggingPrefs", {{"performance": "ALL"}}')
        self.chrome_options.set_capability('goog:loggingPrefs', {'performance': 'ALL'})
        return self

    def enable_notifications(self):
        """Pass the argument 1 to allow and 2 to block

        """
        self.chrome_options.add_experimental_option(
            "prefs", {"profile.default_content_setting_values.notifications": 1}
        )
        logger.debug(
            f'webdriver :: chrome :: '
            f'add_experimental_option :: '
            f'{dict(name="prefs", value={"profile.default_content_setting_values.notifications": 1})}'
        )

        return self

    def enable_maximized(self):
        logger.debug(f'webdriver :: chrome :: add_argument :: --start-maximized')
        self.chrome_options.add_argument('--start-maximized')
        return self

    def enable_proxy(self, proxy: str):
        logger.debug(f'webdriver :: chrome :: add_argument :: --proxy-server')
        self.chrome_options.add_argument(f"--proxy-server={proxy}")
        return self

    def enable_translate(self, native_language: str = 'en'):
        prefs = {
            "translate_whitelists": {"your native language": native_language},
            "translate": {"enabled": "True"}
        }
        self.chrome_options.add_experimental_option(
            name="prefs",
            value=prefs,
        )

        logger.debug(
            f'webdriver :: chrome :: '
            f'add_experimental_option :: '
            f'{dict(name="prefs", value=prefs)}'
        )

        return self

    def close(self):
        """close

        """
        result = self.webdriver.close()
        logger.info(f'webdriver :: chrome :: close :: {result}')
        return result

    def in_docker(self):
        """Chrome best used with docker

        """
        self.enable_defaults()
        self.enable_headless()
        self.disable_sandbox()
        self.disable_infobars()
        self.disable_extensions()
        self.disable_notifications()
        return self

    def in_headless(self):
        """alias to headless sandboxed

        """
        return self.in_headless_sandboxed()

    def in_headless_sandboxed(self):
        """Headless Chrome with sandbox enabled

        """
        logger.warning(
            'Docker does not support sandbox option. '
            'Default shm size is 64m, which will cause chrome driver to crash.'
        )

        self.enable_defaults()
        self.enable_headless()
        return self

    def in_headless_sandbox_disabled(self):
        """Headless Chrome with sandbox disabled

        """
        logger.warning('Default shm size is 64m, which will cause chrome driver to crash.')

        self.enable_defaults()
        self.enable_headless()
        self.disable_sandbox()
        return self

    def in_headless_sandbox_disabled_certificate_unverified(self):
        """Headless Chrome with sandbox disabled with no certificate verification

        """
        logger.warning('Default shm size is 64m, which will cause chrome driver to crash.')

        self.enable_defaults()
        self.enable_headless()
        self.disable_sandbox()
        self.disable_certificate_verification()
        return self

    def in_headless_sandbox_disabled_shm_disabled(self):
        """Headless Chrome with sandbox disabled

        """
        self.enable_defaults()
        self.enable_headless()
        self.disable_sandbox()
        self.disable_shm()
        return self

    def in_headless_sandbox_disabled_bigshm(self):
        """Headless Chrome with sandbox disabled

        """
        logger.warning('Larger shm option is not implemented')

        self.enable_defaults()
        self.enable_headless()
        self.enable_bigshm()
        self.disable_sandbox()
        return self

    def in_remote_driver(self, host: str = '127.0.0.1', port: str = '4444', executor_path: str = '/wd/hub'):
        """Remote Selenium

        """
        logger.info(
            f'Remote WebDriver Hub URL: http://{host}:{port}{executor_path}/static/resource/hub.html')

        selenium.webdriver.Remote(
            command_executor=f'http://{host}:{port}{executor_path}',
            desired_capabilities=selenium.webdriver.common.desired_capabilities.DesiredCapabilities.CHROME
        )
        return self

    def in_sandbox(self):
        """Chrome with sandbox enabled

        """
        logger.warning(
            'Docker does not support sandbox option. '
            'Default shm size is 64m, which will cause chrome driver to crash.'
        )

        self.enable_defaults()
        return self

    def in_sandbox_disabled(self):
        """Chrome with sandbox disabled

        """
        logger.warning('Default shm size is 64m, which will cause chrome driver to crash.')

        self.enable_defaults()
        self.disable_sandbox()
        return self

    def run(self) -> bool:
        try:
            self.update_paths(self.chromedriver_path)

            if self.chromedriver_path:
                self._ChromeService = selenium.webdriver.ChromeService(
                    executable_path=self.chromedriver_path,
                    service_args=self.service_args
                )

                logger.debug(f'webdriver :: chrome :: run :: {self.ChromeService=}')

                self._webdriver = selenium.webdriver.Chrome(
                    service=self.ChromeService,
                    options=self.chrome_options
                )

                if self._enable_antibot_detection:
                    self.disable_javascript_webdriver_prop()

                logger.debug(f'webdriver :: chrome :: run :: {self=}')

                logger.info(f'webdriver :: chrome :: run :: done')
                return True

            self._webdriver = selenium.webdriver.Chrome(options=self.chrome_options)
            logger.info(f'webdriver :: chrome :: run :: {self=}')

            return True
        except Exception as exception:
            raise Exception(f'webdriver :: chrome :: run :: failed :: {exception=}')

    @property
    def service_args(self):
        return self._service_args

    def set_logging_level(self, level: str = 'ALL'):
        """There are 6 available log levels: ALL, DEBUG, INFO, WARNING, SEVERE, and OFF.
        Note that --verbose is equivalent to --log-level=ALL and --silent is equivalent
        to --log-level=OFF, so this example is just setting the log level generically

        """
        logger.debug(f'webdriver :: chrome :: service_args :: {level}')
        self.service_args.append(f'--log-level={level.upper()}')
        return self

    def set_chromedriver(self, chromedriver_path: str):
        logger.debug(f'{chromedriver_path}')
        self._chromedriver_path.append(chromedriver_path)
        self.update_paths(chromedriver_path)
        return self

    def set_locale(self, locale: str = 'en'):
        logger.debug(f'webdriver :: chrome :: add_argument :: "--lang={locale}"')
        self.chrome_options.add_argument(f"--lang={locale}")
        return self

    def set_locale_experimental(self, locale: str = 'en-US'):
        self.chrome_options.add_experimental_option(
            name='prefs',
            value={'intl.accept_languages': locale}
        )

        logger.debug(
            f'webdriver :: chrome :: '
            f'add_experimental_option :: '
            f"{dict(name='prefs', value={'intl.accept_languages': locale})}"
        )

        return self

    def set_user_agent(self, user_agent: str):
        logger.debug(f'webdriver :: chrome :: add_argument :: f"user-agent={user_agent}"')
        self.chrome_options.add_argument(f"user-agent={user_agent}")
        return self

    def set_window_size(self, *args, **kwargs):
        """has to be set after setting webdriver"""
        logger.debug(f'webdriver :: chrome :: set_window_size :: {args=} :: {kwargs=}')
        self._window_size = set_window_size(*args, **kwargs)
        width, height = self.window_size
        self.webdriver.set_window_size(width=width, height=height)
        logger.info(f'webdriver :: chrome :: set_window_size :: {width=} {height=}')
        return self

    def start(self):
        """alias to run

        """
        return self.run()

    def stop_client(self):
        """stop client

        """
        result = self.webdriver.stop_client()
        logger.info(f'{result}')
        return result

    def update_paths(self, path: str):
        if path and os.path.exists(path):
            if path not in os.getenv('PATH'):
                if sys.platform == 'win32':
                    os.environ['PATH'] = f"{os.getenv('PATH')};{path}"
                else:
                    os.environ['PATH'] = f"{os.getenv('PATH')}:{path}"

                # logger.debug(f'update_paths :: {path=} :: {os.environ['PATH']}')
                logger.debug(f'webdriver :: chrome :: update_paths :: {path}')

                logger.info(f'webdriver :: chrome :: update_paths :: done')
                return True

        logger.error(f'webdriver :: chrome :: update_paths :: failed :: {path} not found')
        return False

    def quit(self):
        """quit

        """
        result = self.webdriver.quit()
        logger.info(f'webdriver :: chrome :: quit :: {result}')
        return result

    def quit_gracefully(self):
        """gracefully quit webdriver

        """
        try:
            self.close()
            self.quit()
            self.stop_client()
        except Exception as error:
            logger.error(f'webdriver :: chrome :: quit :: error :: failed to gracefully quit. {error}')
            return False
        return True
